/*
 * Copyright (c) 2005 Kirk Murrin, <mailto:linuxguru1968@yahoo.com>
 * Licensed under the Gnu Public License, version 2.
 * All rights resered.
 * 
 */
// http://www.cs.columbia.edu/~hgs/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "base64.h"

static char map(char c)
{
  if (c >= 0 && c <= 25) return(c + 'A');
  if (c >= 26 && c <= 51) return(c - 26 + 'a');
  if (c >= 52 && c <= 61) return(c - 52 + '0');
  if (c == 62) return('+');
  if (c == 63) return('/');
  else return(-1);
} /* map */


static char unmap(char c)
{
  if (c >= 'A' && c <= 'Z') return(c - 'A');
  if (c >= 'a' && c <= 'z') return(c + 26 - 'a');
  if (c >= '0' && c <= '9') return(c + 52 - '0');
  if (c == '+') return(62);
  if (c == '/') return(63);
  else return(-1);
} /* unmap */


char *base64_decode_string(char *string)
{
  int    length, i, j,len;

  char    *buf;

  if (!string || !*string) return(strdup(""));

  length = strlen(string);

  //buf = (char *) malloc(((length/4)*3) + 1);
  //*buf = 0;
  len=(((length/4)*3) + 1);
  buf=(char *)malloc(len);
  memset(buf,0x0,len);

  for (i=j=0; i<length; i+=4, j+=3) {
    while (i < length && string[i] != '=' &&
           !(string[i] >= 'a' && string[i] <= 'z') &&
           !(string[i] >= 'A' && string[i] <= 'Z') &&
           !(string[i] >= '0' && string[i] <= '9')) i++;

    buf[j] = (unmap(string[i]) & 0x3f) << 2;

    if (string[i+1] == '=') {
      buf[j+1] = 0;
      break;
    }
    buf[j] |= (unmap(string[i+1]) & 0x30) >> 4;

    buf[j+1] = (unmap(string[i+1]) & 0x0f) << 4;

    if (string[i+2] == '=') {
      buf[j+2] = 0;
      break;
    }
    buf[j+1] |= (unmap(string[i+2]) & 0x3c) >> 2;

    buf[j+2] = (unmap(string[i+2]) & 0x03) << 6;

    if (string[i+3] == '=') {
      buf[j+3] = 0;
      break;
    }
    buf[j+2] |= unmap(string[i+3]) & 0x3f;
  }

  return(buf);
} /* base64_decode_string */


char * base64_encode_string(char *string)
{
  int    BLOCKS_PER_LINE = 18;
  int    length, i, j, cnt,len;

  char    *buf;

  if (!string || !*string) return(strdup(""));

  length = strlen(string);

  cnt = (((length + 3) / 3) * 4) + 2;
  cnt += (cnt / (BLOCKS_PER_LINE * 4));

  //buf = (char *) malloc(sizeof(char) * cnt);

  len=(sizeof(char) * cnt);
  buf=(char *) malloc(len);
  memset(buf,0x0,len);

  for (i=j=cnt=0; i<length+3; i+=3,j+=4) {
    if (!string[i]) {
      break;
    }
    buf[j] = map((char) ((string[i] & 0xfc) >> 2));
    buf[j+1] = map((char) (((string[i] & 0x03) << 4) |
             ((string[i+1] & 0xf0) >> 4)));

    if (!string[i+1]) {
      buf[j+2] = buf[j+3] = '=';
      j+=4;
      break;
    }

    buf[j+2] = map((char) (((string[i+1] & 0x0f) << 2) |
             ((string[i+2] & 0xc0) >> 6)));

    if (!string[i+2]) {
      buf[j+3] = '=';
      j+=4;
      break;
    }

    buf[j+3] = map((char) (string[i+2] & 0x3f));

    if (cnt >= (BLOCKS_PER_LINE - 1))
      buf[j+4] = '\n', buf[j+5] = ' ', j += 2, cnt = 0;
    else cnt++;
  }

  buf[j] = 0;

  return(buf);
} /* base64_encode_string */
