/*
 * Copyright (c) 2005 Kirk Murrin, <mailto:linuxguru1968@yahoo.com>
 * Licensed under the Gnu Public License, version 2.
 * All rights resered.
 * 
 */

#include "conf_page.h"
#include "html.h"
#include "response.h"
#include "session.h"
#include "request.h"

extern html_config_type page_config;
extern sys_config_type sys_config;

#define CONF_PAGE_ACTION_SAVE       "Save"
#define CONF_PAGE_ACTION_DEFAULTS   "Defaults"
#define CONF_PAGE_ACTION_PREVIOUS   "Previous"

page_type main_conf_page = {
        "Configure Kernel",
        "&nbsp;",
        "&nbsp;",
        handle_main_conf, 
        {
             INPUT,   { "submit",    PARAM_ACTION, CONF_PAGE_ACTION_SAVE,         },
             INPUT,   { "submit",    PARAM_ACTION, CONF_PAGE_ACTION_DEFAULTS,     },
             INPUT,   { "submit",    PARAM_ACTION, CONF_PAGE_ACTION_PREVIOUS,     },
             NONE,    { "",          "",       "",             }
        }
};

int handle_main_conf( void )
{
    int ret=0;
    char pstr[16];
    char *target,*method;

    target = get_request_target();
    method = get_request_method();

    set_response_title(NULL);
    if ( strcasecmp(method,"GET") == 0 ) {
        if ( target != NULL ) {
            if ( session_set_statement(target) ) {
                ret=process_pointer();
            } else {
                setup_error(RC_INTERNAL_SERVER_ERROR,
                        "Requested URI pointer %s not in proper format",target);
            }
        } else {
            session_set_statement_block_first();
        }
    } else {
        if ( strcasecmp(method,"POST") == 0 ) {
            ret=process_conf_post();
        } else {
            setup_error(RC_METHOD_NOT_ALLOWED,
                        "The method specified in the Request-Line is  not allowed for \
the resource identified by the Request-URI: %s",method);
        }
    }

    if ( ret == 0 ) {
       setup_conf_menu();
    } 

    return SEND_PAGE;
}

/*
 * Process the HTTP Request 
 * Returns:
 *   -1   did not handle command
 *    0   handled request
 */

int process_pointer(void)
{
	//mstate_type  *mstate = Session->mstate;
        session_type *session;

        session =  get_session();

	switch ((session->statement)->verb) {
	default:
        {
            setup_error(RC_NOT_FOUND,"The server has not found anything matching the Request-URI: %p.",session->statement);
        }
        return 0;
	case verb_MENU:
	{
                push_statement();
		return 0;
	}
	break;

	case verb_comment:
	case verb_text:
		return -1;

	case verb_ask_bool:
	case verb_dep_bool:
	case verb_ask_tristate:
	case verb_dep_tristate:
	case verb_dep_mbool:
	{
                // Rotate through allowed values
		const int   allowed = statement_bools_allowed(session->statement);
		piece_type  value = session->statement->sb_symbol->value;
                switch(value.ptr[0]) {
                	case '\0':
                        	if ( allowed & M_ALLOWED ){
				     session->statement->sb_symbol->value = piece_m;
                                 }
                                 else {
                                     if ( allowed & Y_ALLOWED ){
				         session->statement->sb_symbol->value = piece_y;
                                     }
                                     else  {
				        session->statement->sb_symbol->value = piece_empty;
                                     }
                                }
                        break;
                        case 'y':
                                    if ( allowed & N_ALLOWED ){
				        session->statement->sb_symbol->value = piece_n;
                                    } else {
                                        if ( allowed & M_ALLOWED ){
				            session->statement->sb_symbol->value = piece_m;
                                         } else {
				            session->statement->sb_symbol->value = piece_empty;
                                        }
                                   }
                        break;
                        case 'n':
                                 if ( allowed & M_ALLOWED )
				     session->statement->sb_symbol->value = piece_m;
                                 else {
                                     if ( allowed & Y_ALLOWED ){
				         session->statement->sb_symbol->value = piece_y;
                                     } else  {
				        session->statement->sb_symbol->value = piece_empty;
                                    }
                                 }
                        break;
                        case 'm':
                                if ( allowed & Y_ALLOWED ){
				    session->statement->sb_symbol->value = piece_y;
                                } else {
                                    if ( allowed & N_ALLOWED ){
				        session->statement->sb_symbol->value = piece_n;
                                    } else {
				         session->statement->sb_symbol->value = piece_empty;
                                    }
                                }
                         break;

                }    
		session->statement->sb_symbol->origin = origin_statement;
		return 0;
	}
	break;
/*
	case verb_ask_hex:
	case verb_dep_hex:
	case verb_ask_int:
	case verb_dep_int:
	case verb_ask_string:
	case verb_dep_string:
	{
	}
	break;
*/
	case verb_nchoice:
	{
		symbol_type  *symbol;
		int          iline;
	    	// increment the current choice 
		session->statement->sn_choice_index =
			(session->statement->sn_choice_index + 1) % session->statement->sn_choice_count;
		// set the symbols appropriately 
		for (symbol = session->statement->sn_choice_list->symbol_first, iline = 0; 
                     symbol != NULL; symbol = symbol->next, iline++) {
			symbol->value = (iline == session->statement->sn_choice_index)
					? piece_y : piece_n;
					symbol->origin = origin_statement;
		}
	    	return 0;

	}
	break;
	}
	return -1;
}

int process_conf_post( void )
{
   int i,ret;
   mstate_type  *mstate;
   char defconfig_file[128];
   char buf[16],*action;
   session_type *session;

   session = get_session();
   mstate = session->mstate;

   action = get_request_action();

   ret=0;
   if ( strcmp(action,CONF_PAGE_ACTION_SAVE) == 0 ) {
       process_params();
       commit_changes();
   } else {
       if ( strcmp(action,CONF_PAGE_ACTION_DEFAULTS) == 0 ) {
           memset(defconfig_file,0x0,sizeof(defconfig_file)); 
           sprintf(defconfig_file, "%sarch/%s/defconfig",
	        argument.ds, argument.arch);
           load_defconfig(session->block,defconfig_file);
       } else {
           if ( strcmp(action,CONF_PAGE_ACTION_PREVIOUS) == 0 ) {
                if (  mstate->parent != NULL ) {
                    pop_statement();
                    if ( !is_top_menu() ) {
                        memset(buf,0x0,sizeof(buf)); 
                        sprintf(buf,"%p",session->mstate->menu);
                        set_request_target(buf);
                    }
                }
           }
       }
   }
   return ret;
}


/*
 * Render one statement.
 */

void statement_render(const statement_type *statement, void *param_mstate)
{
        int len=0;

	switch (statement->verb) {
	default:
                //start_row_col();
                start_row_new(page_config.table_row_height);
                append_page_data_body("Invalid menu statement pointer %p",statement);
                end_row_col();
		return;

	case verb_MENU:
		{
                //start_row();
                //start_row_new(page_config.table_row_height);
                //start_col();
                //start_col_new(page_config.table_col_width);
                start_row_col_new(page_config.table_row_height,page_config.table_col_width);
                start_pointer_link(statement);

                append_page_data_body("%*.*s -->&nbsp;",
                     statement->sc_title->value.len,
                     statement->sc_title->value.len,
                     statement->sc_title->value.ptr);

                end_link();
                end_col();
                //add_help_link(statement->sb_symbol);
                end_row();
		}
		return;

	case verb_mainmenu_name:
		/* ignore this statement */
		return;

	case verb_comment:
                {
                char buf[128];
		piece_type      prompt = statement->sb_prompt->value;
                //start_row_col();
                start_row_col_new(page_config.table_row_height,page_config.table_col_width);
                append_page_data_body("---&nbsp;%*.*s",prompt.len,prompt.len,prompt.ptr);
                memset(buf,0x0,sizeof(buf));
                append_page_data_body("%*.*s",prompt.len,prompt.len,prompt.ptr);
                sprintf(buf,"%*.*s",prompt.len,prompt.len,prompt.ptr);
                set_response_title(buf);
                end_row_col();
                }
		return;

	case verb_text:
                {
		piece_type      prompt = statement->sb_prompt->value;
                //start_row_col();
                start_row_col_new(page_config.table_row_height,page_config.table_col_width);
                start_pointer_link(statement);
                append_page_data_body("%*.*s",prompt.len,prompt.len,prompt.ptr);
                end_link();
                end_row_col();
                }
		return;

	case verb_unset:
		{
			symbol_type    *symbol;

			for (symbol = statement->su_unset_list->symbol_first;
			     symbol != NULL;
			     symbol = symbol->next) {
				symbol->value = piece_empty;
				symbol->origin = origin_unset;
			}
		}
		return;

	case verb_def_bool:
	case verb_def_hex:
	case verb_def_int:
	case verb_def_string:
	case verb_def_tristate:
		statement->sb_symbol->value = word_eval(statement->sb_def_value, 1);
		statement->sb_symbol->origin = origin_statement;
		return;

	case verb_ask_bool:
	case verb_dep_bool:
	case verb_dep_mbool:
		if ((statement_bools_allowed(statement)&~N_ALLOWED) == 0)
			return;
                 { 
			piece_type      prompt = statement->sb_prompt->value;
			piece_type      value = statement->sb_symbol->value;
                        //start_row();
                        //start_row_new(page_config.table_row_height);
                        //start_col();
                        //start_col_new(page_config.table_col_width);
                        start_row_col_new(page_config.table_row_height,page_config.table_col_width);
                        start_pointer_link(statement);
			if (value.len == 1 && value.ptr[0] == 'y') {
                                append_page_data_body("[*]&nbsp;");

			} else {
                                append_page_data_body("[&nbsp;]&nbsp;");

				/*
				 * force value to be 'n', whatever it was
				 * before
				 */
				statement->sb_symbol->value = piece_n;
			}

                        append_page_data_body("%*.*s",prompt.len,prompt.len, prompt.ptr);

			if (statement->sb_symbol->origin == origin_unset){
                                append_page_data_body("&nbsp;(NEW)");
                        }
                        end_link();
                        end_col();
                        add_help_link(statement->sb_symbol);
                        end_row();
                }
		return;

	case verb_ask_tristate:
	case verb_dep_tristate:
		if ((statement_bools_allowed(statement)&~N_ALLOWED) == 0)
			return;
		{
		    piece_type      prompt = statement->sb_prompt->value;
		    piece_type      value = statement->sb_symbol->value;
                    //start_row();
                    //start_row_new(page_config.table_row_height);
                    //start_col();
                    //start_col_new(page_config.table_col_width);
                    start_row_col_new(page_config.table_row_height,page_config.table_col_width);
                    start_pointer_link(statement);
		    if (value.len == 1 && value.ptr[0] == 'y'){
                                append_page_data_body("&lt;*&gt;&nbsp;");

                    } else if (value.len == 1 && value.ptr[0] == 'm'){
                                append_page_data_body("&lt;m&gt;&nbsp;");

                    } else {
                                append_page_data_body("&lt;&nbsp;&gt;&nbsp;");
				/*
				 * force value to be 'n', whatever it was
				 * before
				 */
				statement->sb_symbol->value = piece_n;
		    }
                    append_page_data_body("%*.*s",prompt.len,prompt.len, prompt.ptr);

		    if (statement->sb_symbol->origin == origin_unset){
                        append_page_data_body("&nbsp;(NEW)");
                     }
                     end_link();
                     end_col();
                     add_help_link(statement->sb_symbol);
                     end_row();
		}
		return;

	case verb_ask_hex:
	case verb_dep_hex:
	case verb_ask_int:
	case verb_dep_int:
	case verb_ask_string:
	case verb_dep_string:
		if ((statement_bools_allowed(statement)&~N_ALLOWED) == 0)
			return;

		{
			piece_type      prompt = statement->sb_prompt->value;
			piece_type      value = statement->sb_symbol->value;

			/* first the header line */
			{
                                //start_row_col();
                                start_row_col_new(page_config.table_row_height,page_config.table_col_width);
                                append_page_data_body("...&nbsp;");

                                append_page_data_body("%*.*s",prompt.len,prompt.len,prompt.ptr);

				if (statement->sb_symbol->origin == origin_unset) {

                                        append_page_data_body("&nbsp;(NEW)");

					/*
					 * this should be a new kind of
					 * origin
					 */
					value = word_eval(statement->sb_def_value, 0);
					statement->sb_symbol->value = value;
				}
                                end_row_col();
			}

			/* then the data line */
			{
                                
                                //start_row();
                                //start_col();
                                //start_row_new(page_config.table_row_height);
                                //start_col_new(page_config.table_col_width);
                                start_row_col_new(page_config.table_row_height,page_config.table_col_width);
                                append_page_data_body("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");

                                if (statement->verb == verb_ask_hex
                                    || statement->verb == verb_dep_hex){
                                    append_page_data_body("0x");
                                }

                                append_page_data_body("<input type=\"text\" name=\"%p\"",statement);

                                append_page_data_body("size=\"%d\" value=\"",
                                    value.len+((statement->verb == verb_ask_hex || statement->verb == verb_dep_hex)?2:0));

                                append_page_data_body("%*.*s",value.len,value.len,value.ptr);

                                append_page_data_body("\"></input>");

                                if ( statement->err_mess != NULL ) {
                                    append_page_data_body("&nbsp;&nbsp;&nbsp;&nbsp;<blink><font color=#red>%s</font></blink>",statement->err_mess);
                                }
                                end_col();
                                end_row();
			}
		}
		return;

	case verb_nchoice:
		{
			const prompt_type *prompt;
			symbol_type    *symbol;
			int             is_new = 0;

			/*
		         * Count the header line.
		         *
		         * I can't render it yet because its "(NEW)" status depends on the
		         * origins of all the lines below it.
		         */

			for (prompt = statement->sn_choice_list->prompt_first,
			   symbol = statement->sn_choice_list->symbol_first;
			     prompt != NULL && symbol != NULL;
			     prompt = prompt->next, symbol = symbol->next) {
				if (symbol->origin == origin_unset)
					is_new = 1;
                        }

			/*
		         * Render the header line first
		         */
                        //start_row_col();
                        start_row_col_new(page_config.table_row_height,page_config.table_col_width);
                        append_page_data_body("###&nbsp;");

                        append_page_data_body("%*.*s",
				 statement->sn_prompt->value.len,
				 statement->sn_prompt->value.len,
				 statement->sn_prompt->value.ptr);

			if (is_new){
                            append_page_data_body("&nbsp;(NEW)");
                        }
                        end_row_col();

			/*
		         * Render subordinate lines.
		         */

			for (prompt = statement->sn_choice_list->prompt_first,
			   symbol = statement->sn_choice_list->symbol_first;
			     prompt != NULL && symbol != NULL;
			     prompt = prompt->next, symbol = symbol->next) {

				if (symbol->origin == origin_unset)
					is_new = 1;

                                //start_row();
                                //start_col();
                                //start_row_new(page_config.table_row_height);
                                start_row_col_new(page_config.table_row_height,page_config.table_col_width);

                                append_page_data_body("&nbsp;&nbsp;&nbsp;&nbsp;");

                                start_pointer_link(statement);

				if (symbol->value.len == 1 && symbol->value.ptr[0] == 'y') {
                                    append_page_data_body("[*]&nbsp;");

                                } else {
                                    append_page_data_body("[&nbsp;]&nbsp;");
					/*
					 * force value to be 'n',
					 * whatever it was before
					 */
				     symbol->value = piece_n;
				}

                                append_page_data_body("%*.*s",prompt->value.len,prompt->value.len,prompt->value.ptr);

                                end_link();
                                end_col();
                                //add_help_link(statement->sb_symbol);
                                add_help_link(symbol);
                                end_row();
			}
			return;
                }
		break;

	}
}

int setup_conf_menu(void) 
{
    session_type *session;

    session = get_session();

    start_content();
    start_div(page_config.max_textarea_cols*
              page_config.char_pixel_width,
              page_config.max_textarea_rows*
              page_config.char_pixel_height);
    start_table(1,0,0,0,page_config.table_bg_color,0);
    block_walk(session->mstate->menu->sc_block_left, 0, statement_render,session->mstate);
    end_table();
    end_div();
    end_content();
    start_page();
    start_body();
    end_body();
    end_page();
}

int commit_changes(void)
{
	int            written;
	const char     *str_error;
	char         *fco = (char*)argument.fco;
	char           buf[75];
        int retcode,len;
	argument.fco = ".config";

        retcode=0;

        if ( get_response_error_found() != 0 ) return retcode;

	//if (argument.fco == 0 || strcmp(argument.fco, ".config") != 0) {
		written = block_output(session_get_block_top(), &str_error);
		if (written < 0) {
                        // Write failed
			len=sprintf(buf,"Write failed: %s", str_error);
                        retcode=-1;
		} else {
                        // Write successful
			argument.fco = fco;
			retcode=1;
		}
		argument.fco = fco;
        //}
        return retcode;
}


int save_config(void)
{
    //mstate_type  *mstate = MState;
    //mstate_type  *mstate = Session->mstate;
    //const statement_type *statement = *pstatement;
    int ret=0;
    //int sd = Connection.client_sd;

    commit_changes();
    //FIXME: Do I really need to do this?? Put it back if probelm
     session_set_statement_block_first();
    //Session->statement=Session->block->first;

     //member functions???
    //free_string(&Request.uri);
    //free_string(&Request.target);
    //mstate = block;
   return ret;
}
