/*
 * Copyright (c) 2005 Kirk Murrin, <mailto:linuxguru1968@yahoo.com>
 * Licensed under the Gnu Public License, version 2.
 * All rights resered.
 * 
 */

#include <stdio.h>
#include <fcntl.h>
#include <syslog.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>

#include "config.h"

#include "mconfig.h"
#include "connection.h"
#include "response.h"

// The current connection to client
connection_type Connection;

extern int errno;

int init_server( void )
{
    int ret=0;
    ret=create_socket();
    if ( ret < 0 ) {
        exit(ret);
    }

    //setnonblocking();

    ret=bind_socket();
    if ( ret < 0 ) {
        exit(ret);
    }

    ret=connection_listen();
    if ( ret < 0 ) {
        exit(ret);
    }
    return ret;
}

void print_connection( void )
{
    printf("----------------------------------------------------------\n");
    printf("server socket desc: %d\n",Connection.server_sd);
    printf("client socket desc: %d\n",Connection.client_sd);
    printf("addr len: %d\n", Connection.addr_len);
    printf("Server address: %s\n",inet_ntoa(Connection.server_addr.sin_addr));
    printf("Server port: %d\n", htons(Connection.server_addr.sin_port));
    printf("Client address: %s\n",inet_ntoa(Connection.client_addr.sin_addr));
    printf("Client port: %d\n", htons(Connection.client_addr.sin_port));
    printf("----------------------------------------------------------\n");
}

void clear_connection( void )
{
    Connection.server_sd=-1;
    Connection.client_sd=-1;
    Connection.addr_len=0;
    memset(&Connection.server_addr,0x0,sizeof(Connection.server_addr));
    memset(&Connection.client_addr,0x0,sizeof(Connection.client_addr));
    Connection.maxfd=0;
    memset(&Connection.readfds,0x0,sizeof(Connection.readfds));
    memset(&Connection.writefds,0x0,sizeof(Connection.writefds));
    memset(&Connection.exceptfds,0x0,sizeof(Connection.exceptfds));

}

void setnonblocking(void)
{
	int opts;
        int sock = Connection.server_sd;

	opts = fcntl(sock,F_GETFL);
	if (opts < 0) {
		perror("fcntl(F_GETFL)");
		exit(EXIT_FAILURE);
	}
	opts = (opts | O_NONBLOCK);
	if (fcntl(sock,F_SETFL,opts) < 0) {
		perror("fcntl(F_SETFL)");
		exit(EXIT_FAILURE);
	}
	return;
}

int create_socket( void ) 
{
    Connection.server_sd = socket(AF_INET,SOCK_STREAM, 0);
    if(Connection.server_sd < 0) {
        syslog(LOG_ERR|LOG_USER,"do_mode_menu: cannot open socket: %s",
                   strerror(errno));
        perror("cannot open socket ");
        return -1;
    }
    syslog(LOG_ERR|LOG_USER,"do_mode_menu: opened server socket: fd=%d",Connection.server_sd);
    return 0;
}

int bind_socket( void )
{
    Connection.server_addr.sin_family = AF_INET;
    Connection.server_addr.sin_addr.s_addr = htonl(INADDR_ANY);
    //Connection.server_addr.sin_port = htons(argument.port);
    Connection.server_addr.sin_port = htons(Connection.server_port);
    if (bind(Connection.server_sd, (struct sockaddr *) &Connection.server_addr, 
            sizeof(Connection.server_addr)) < 0 ) {
        char err_mess[75];
        sprintf(err_mess,"Cannot bind on port %d",argument.port);
        perror(err_mess);
        syslog(LOG_ERR|LOG_USER,"do_mode_menu: cannot bind port: %s",
               strerror(errno));
        return -1;
    }
    syslog(LOG_ERR|LOG_USER,"do_mode_menu: bind succesful");
    return 0;
}

int accept_connection( void )
{
 
    Connection.addr_len = sizeof(Connection.client_addr);
    Connection.client_sd = accept(Connection.server_sd,
                                 (struct sockaddr*)&Connection.client_addr,
                                 &Connection.addr_len);

    syslog(LOG_ERR|LOG_USER,"Accepted connection: %d\n",
            Connection.client_sd);

    if( Connection.client_sd < 0 ) {
        char err_mess[75];
        syslog(LOG_ERR|LOG_USER,"do_mode_menu: accept failed: %s",
            strerror(errno));
        sprintf(err_mess,"Cannot accept connection on port %d",argument.port);
        perror(err_mess);
        cleanup();
        return -1 ;
    }

    FD_ZERO(&Connection.readfds);
    FD_ZERO(&Connection.writefds);
    FD_ZERO(&Connection.exceptfds);
    FD_SET(Connection.client_sd,&Connection.readfds);
    Connection.maxfd = Connection.client_sd+1;
    //Connection.timeout.tv_sec=SEL_TIMEOUT;
    Connection.timeout.tv_sec=0;
    Connection.timeout.tv_usec=0;

    //print_connection();

}

int check_connection( void )
{
    syslog(LOG_ERR|LOG_USER,"check_connection: start");
    if ( get_response_connection_close() || get_response_protocol_ver() <= 1.0 ) { 
       close(Connection.client_sd);
       syslog(LOG_ERR|LOG_USER,"check_connection: %d closed",Connection.client_sd);
       return 1;
    }
    syslog(LOG_ERR|LOG_USER,"check_connection: connection still open");
    return 0;
}

connection_type *get_connection( void )
{
    return &Connection;
}

int get_connection_client_sd( void )
{
    return Connection.client_sd;
}

int set_connection_client_sd( int sd )
{
    Connection.client_sd = sd ;
}

int get_connection_server_sd( void )
{
    return Connection.server_sd;
}

int set_connection_server_sd( int sd )
{
    Connection.server_sd = sd ;
}

void connection_close_client( void )
{
    close(Connection.client_sd);
}

void connection_close_server( void )
{
    close(Connection.server_sd);
}

int connection_listen( void )
{
    return listen(Connection.server_sd,5);
}

long connection_get_connection_s_addr( void )
{
    return Connection.client_addr.sin_addr.s_addr;
}

struct in_addr connection_get_connection_sin_addr( void )
{
    return Connection.client_addr.sin_addr;
}

int get_connection_server_port( void )
{
    return Connection.server_port;
}

int set_connection_server_port( int portnum )
{
    Connection.server_port = portnum;
}

