/*
 * Copyright (c) 2005 Kirk Murrin, <mailto:linuxguru1968@yahoo.com>
 * Licensed under the Gnu Public License, version 2.
 * All rights resered.
 * 
 */

#include <string.h>
#include <sys/types.h>
#include <dirent.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <time.h>
#include <pwd.h>
#if !defined(USE_PAM)
#include <shadow.h>
#include <pwauth.h>
#endif
#include <grp.h>
#include <sys/types.h>
#include <libgen.h>

#include "mconfig.h"
#include "response.h"
#include "vv_array.h"
#include "html.h"
#include "session.h"
#include "request.h"
#include "page_util.h"

#include "edit_page.h"

extern int errno;
extern html_config_type page_config;
extern sys_config_type sys_config;

#define EDIT_PAGE_ACTION_SAVE    "Save"
#define EDIT_PAGE_ACTION_RELOAD  "Reload"
#define EDIT_PAGE_ACTION_DELETE  "Delete"
#define EDIT_PAGE_ACTION_APPEND  "Append"
#define EDIT_PAGE_ACTION_RENAME  "Rename"
#define EDIT_PAGE_ACTION_SAVE_AS "Save As"
#define EDIT_PAGE_ACTION_DISCARD "Discard"

#define EDIT_PAGE_PARAM_RENAMEFILE "renamefile"
#define EDIT_PAGE_PARAM_SAVEASFILE "saveasfile"

page_type main_edit_file_page= {
        "Edit Filesystem",
        "&nbsp;",
        "&nbsp;",
        handle_edit_file, 
        {
             INPUT,   { "submit",    PARAM_ACTION, EDIT_PAGE_ACTION_SAVE,    },
             INPUT,   { "submit",    PARAM_ACTION, EDIT_PAGE_ACTION_RELOAD,  },
             INPUT,   { "submit",    PARAM_ACTION, EDIT_PAGE_ACTION_DELETE,  },
             INPUT,   { "submit",    PARAM_ACTION, EDIT_PAGE_ACTION_APPEND,  },
             INPUT,   { "submit",    PARAM_ACTION, EDIT_PAGE_ACTION_RENAME,  },
             INPUT,   { "text",      EDIT_PAGE_PARAM_RENAMEFILE, "new.c",  "12",  24, },
             INPUT,   { "submit",    PARAM_ACTION, EDIT_PAGE_ACTION_SAVE_AS, },
             INPUT,   { "text",      EDIT_PAGE_PARAM_SAVEASFILE, "new.c",  "12",  24, },
             NONE,    { "",          "",       "",        }
        }
};

#define EDIT_PAGE_ACTION_CREATE_NEW_FILE  "Create New File"
#define EDIT_PAGE_PARAM_NEWFILE "newfile"

page_type  main_edit_dir_page = {
        "Edit Filesystem",
        "&nbsp;",
        "&nbsp;",
        handle_edit_dir, 
        {
             INPUT,   { "submit",    PARAM_ACTION,   EDIT_PAGE_ACTION_CREATE_NEW_FILE, },
             LABEL,   { "New File Name:",                     },
             INPUT,   { "text",      EDIT_PAGE_PARAM_NEWFILE, "new.c", "12", 24, },
             NONE,    { "",          "",          "",               }
        }
};

//Table used to filter chars in file text that can cause problems
// when displayed in textarea

static html_filter_char filter_table[]=
{
     '<', "&#060;",
     '>', "&#062;",
     '"', "&#034;"

     /* ' ', "&nbsp;",
     '-', "&#183;",
     '/', "&#047;",
     '!', "&#033;",
     '%', "&#037;"
     */

};

#define sizeof_filter_table (sizeof(filter_table)/sizeof(filter_table[0]))

int handle_edit_file( void )
{
   int len,fret;
   char pathname[256],newpath[256],*newname=NULL;
   char *newfile=NULL,*target=NULL;
   char *method=NULL,*action=NULL,*cwd=NULL;
   request_type *request=NULL;
   page_status_type ret;
  
   ret = SEND_PAGE;

   request = get_request();
   target = request->target;
   method = request->method;
   action = request->action;
   cwd=session_get_cwd();

   //FIXME: Test for valid file ?? do I need a null default???
   memset(pathname,0x0,sizeof(pathname));
   if ( target  == NULL ) {
       strcpy(pathname,"/");
   } else {
       sprintf(pathname,"%s",target);
   }

   set_response_title(pathname);
   //set_request_stat(pathname);
    
   if ( strcasecmp(method,"GET") == 0 ) {
        ret=setup_edit_file(pathname);
   } else {
       if ( strcasecmp(method,"POST") == 0 ) {
           if ( strcmp(action,EDIT_PAGE_ACTION_SAVE) == 0 ){
                   process_files();
                   ret=setup_edit_file(pathname);
           } else {
               if ( strcmp(action,EDIT_PAGE_ACTION_RELOAD) == 0 ){
                   ret=setup_edit_dir(cwd);
               } else {
               if ( strcmp(action,EDIT_PAGE_ACTION_DELETE) == 0 ){
                     ret=removefile(pathname);
                     if ( ret == DONT_SEND ) {
                         set_request_curr_page_by_uri(MAIN_EDIT_DIR_URL);
                         ret=setup_edit_dir(cwd);
                     }
                   } else {
                       if ( strcasecmp(action,EDIT_PAGE_ACTION_DISCARD) == 0 ) {
                           ret=setup_edit_dir(cwd);
                       } else {
                           if ( strcasecmp(action,EDIT_PAGE_ACTION_SAVE_AS) == 0 ) {
                               process_files();
                               ret=copyfile(pathname,&newname);
                               if ( ret == DONT_SEND ){
                                   ret=setup_edit_file(pathname);
                               }
                               free(newname);
                           } else {
                               if ( strcasecmp(action,EDIT_PAGE_ACTION_RENAME) == 0 ) {
                                   process_files();
                                   ret=renamefile(pathname,&newname);
                                   if ( ret == DONT_SEND ) {
                                       ret=setup_edit_file(pathname);
                                   }
                                   free(newname);
                               } else {
                                   if ( strcasecmp(action,EDIT_PAGE_ACTION_APPEND) == 0 ) {
                                       set_response_append(1);
                                       ret=setup_edit_file(pathname);
                                   }
                               }
                           }
                       }
                   }
               }
           }
       }
   }

   return ret;
}

int handle_edit_dir( void )
{
   int ret=0,len;
   char pathname[256], newpath[256];
   FILE *fp;
   char *newfile,*target,*method,*action,*cwd;
   time_t curtime;
   struct tm *gm_time;
   curtime = time (NULL);
   gm_time = gmtime(&curtime);
  
   ret = SEND_PAGE;

   target = get_request_target();
   method = get_request_method();
   action = get_request_action();

   memset(pathname,0x0,sizeof(pathname));
   if ( target == NULL ) {
       strcpy(pathname,"/");
   } else {
       sprintf(pathname,"%s",target);
   }

   session_set_cwd(pathname);

   cwd = session_get_cwd();

   memcpy(pathname,cwd,strlen(cwd));

   set_response_title(pathname);
   //set_request_stat(pathname);


   if ( strcasecmp(method,"GET") == 0 ) {
        ret=setup_edit_dir(pathname);
   } else {
       if ( strcasecmp(method,"POST") == 0 ) {
           if ( strcmp(action,EDIT_PAGE_ACTION_CREATE_NEW_FILE) == 0 ){
               newfile=find_params_value(EDIT_PAGE_PARAM_NEWFILE);
               if ( newfile  != NULL ) {
                  memset(newpath,0x0,sizeof(newpath));
                  sprintf(newpath,"%s/%s",pathname,newfile);
                  fp=fopen(newpath,"w+");
                  if ( fp  != NULL ) {
                      //char buf[128];
                      //time_t curtime;
                      //struct tm *gm_time;
                      //memset(buf,0x0,sizeof(buf));
                      //curtime=time (NULL);
                      //gm_time=gmtime(&curtime);
                      //strftime (buf, sizeof(buf),"%a,%d %b %Y %H:%M:%S GMT",gm_time);
                      //fprintf(fp,"/* Created by %s : %s */\n",argument.title, buf);
                      fclose(fp);
                      ret=setup_edit_dir(pathname);
                  } else {
                         setup_error(RC_INTERNAL_SERVER_ERROR,"Create %s failed: %s",
                                     newpath, strerror(errno));
                         ret=SEND_PAGE;
                  }
                  free(newfile);
               }
           } else {
           }
       }
   }
   return ret;
}

//FIXME: Clean/simplify logic here; use filter table to remove brackets and
// quotoes so that embedded html in source files can be editied.
//Stratagy? Just read file until EOF with block writing logic?? Filter in
// kernel buffers - discard using sendfile???

int setup_edit_file( char *pathname ) 
{
  int len,fd,rc,ret,sret;
  char cwdpath[PATH_MAX];
  long offset,total;
  long sendlen, datalen;
  char line[512];
  int saved_nodelay, nodelay;
  int saved_tcp_cork, tcp_cork;
  socklen_t nodelay_size, cork_size;
  FILE *fp;
  char *buf;
  struct stat m;
  response_type *response;

  memset(&m,0x0,sizeof(m));
  sret = stat(pathname,&m);

  if ( sret < 0 ) {
      ret=setup_error(RC_INTERNAL_SERVER_ERROR,
                      "Cannot get stat info on %s : %s",pathname,strerror(errno));
      return SEND_PAGE;
  }

  getcwd(cwdpath,sizeof(cwdpath));
   
  set_response_title(pathname);

  response = get_response();

  ret = DONT_SEND;
 
  if ( S_ISREG(m.st_mode) ){

          start_inline_send();

          if ( !is_zero_read(pathname) ) {

              clear_response_data_body();
              add_comment("START OF TEXTAREA");
              start_textarea( page_config.max_textarea_rows,
                              page_config.max_textarea_cols,
                              0,pathname);
              send_line("%X\r\n",response->body.datalen);
              rc=send_data(&response->body.data,response->body.datalen);
 
              fd = open(pathname,O_RDONLY);
              if ( fd != -1 ) {
                  offset=0L;
                  datalen = m.st_size;
                  do {
                      //clear_response_data_body();
                      //add_comment("START OF TEXTAREA");
                      //start_textarea( page_config.max_textarea_rows,
                       //           page_config.max_textarea_cols,
                        //          0,pathname);
                      //send_line("%X\r\n",response->body.datalen);
                      //rc=send_data(&response->body.data,response->body.datalen);

                      sendlen = MIN(sys_config.max_textarea_file,datalen);
                      send_line("%X\r\n",sendlen);
                      rc=sendfile_data(fd,&offset,sendlen);
                      datalen -= sendlen;

                      clear_response_data_body();
                      end_textarea();
                      add_comment("END OF TEXTAREA");
                      send_line("%X\r\n",response->body.datalen);
                      rc=send_data(&response->body.data,response->body.datalen);
                      fsync_data();

                      if ( datalen > 0 ) {
                          clear_response_data_body();
                          add_comment("START OF TEXTAREA");
                          start_textarea( page_config.max_textarea_rows,
                              page_config.max_textarea_cols,
                              0,pathname);
                          send_line("%X\r\n",response->body.datalen);
                          rc=send_data(&response->body.data,response->body.datalen);
                      }

                  } while ( datalen > 0 );

                  close(fd);

              } else {
                      clear_response_data_body();
                      end_textarea();
                      add_comment("END OF TEXTAREA");
                      send_line("%X\r\n",response->body.datalen);
                      rc=send_data(&response->body.data,response->body.datalen);
                      fsync_data();
              }

          } else {
             // stat lied; file size zero but has readable content; /proc or /dev ??
             sendlen = sys_config.max_textarea_file;
             fd = open(pathname,O_RDONLY);
             if ( fd != -1 ) {
                buf=malloc(sendlen);
                memset(buf,0x0,sendlen);
                while ( (datalen=read(fd,buf,sendlen)) != 0 ) {
                      clear_response_data_body();
                      add_comment("START OF TEXTAREA");
                      start_textarea( page_config.max_textarea_rows,
                                  page_config.max_textarea_cols,
                                  0,pathname);
                      send_line("%X\r\n",response->body.datalen);
                      rc=send_data(&response->body.data,response->body.datalen);
                      fsync_data();
                    
                      //send the file content
                      send_line("%X\r\n",datalen);
                      rc=send_data(&buf,datalen);
                      fsync_data();

                      clear_response_data_body();
                      end_textarea();
                      add_comment("END OF TEXTAREA");
                      send_line("%X\r\n",response->body.datalen);
                      rc=send_data(&response->body.data,response->body.datalen);
                      fsync_data();

                }
                close(fd);
                free(buf);
             }

          }

          if ( response->append ) {
              clear_response_data_body();
              add_comment("START OF TEXTAREA");
              start_textarea( page_config.max_textarea_rows,
                  page_config.max_textarea_cols,
                  0,pathname);
              end_textarea();
              add_comment("END OF TEXTAREA");
              send_line("%X\r\n",response->body.datalen);
              rc=send_data(&response->body.data,response->body.datalen);
              fsync_data();

          }

           ret = end_inline_send();

  } else {
      ret=setup_error(RC_INTERNAL_SERVER_ERROR,
                      "Request-URI: Cannot edit %s : %s",pathname,strerror(errno));
  } 

  return ret;

}

char *get_perm_str(struct stat *m)
{
   int i;
   long tmpperm;
   char mstr[12];

   i=0;
   memset(mstr,0x0,sizeof(mstr));
   tmpperm = m->st_mode&S_IFMT;
   switch( tmpperm )
   {
       case S_IFSOCK: mstr[i]='s';
       break;
       case S_IFLNK: mstr[i]='l';
       break;
       case S_IFREG: mstr[i]='-';
       break;
       case S_IFBLK: mstr[i]='b';
       break;
       case S_IFDIR: mstr[i]='d';
       break;
       case S_IFCHR: mstr[i]='c';
       break;
       case S_IFIFO: mstr[i]='f';
       break;
       default: mstr[i]='?';
       break;
   }

   tmpperm = m->st_mode&S_IRWXU;
   i++;
   if ( tmpperm&S_IRUSR ) mstr[i]='r'; else  mstr[i]='-';
   i++;
   if ( tmpperm&S_IWUSR ) mstr[i]='w'; else  mstr[i]='-';
   i++;
   if ( tmpperm&S_IXUSR ) mstr[i]='x'; else  mstr[i]='-';


   tmpperm = m->st_mode& S_IRWXG;
   i++;
   if ( tmpperm&S_IRGRP ) mstr[i]='r'; else  mstr[i]='-';
   i++;
   if ( tmpperm&S_IWGRP ) mstr[i]='w'; else  mstr[i]='-';
   i++;
   if ( tmpperm&S_IXGRP ) mstr[i]='x'; else  mstr[i]='-';


   tmpperm = m->st_mode&S_IRWXO;
   i++;
   if ( tmpperm&S_IROTH ) mstr[i]='r'; else  mstr[i]='-';
   i++;
   if ( tmpperm&S_IWOTH ) mstr[i]='w'; else  mstr[i]='-';
   i++;
   if ( tmpperm&S_IXOTH ) mstr[i]='x'; else  mstr[i]='-';

   return strdup(mstr);

}


int setup_edit_dir( char *pathname ) 
{
   int sret,ret;
   DIR *dir;
   struct stat m;
   struct dirent *dirent=NULL;
   char fullname[NAME_MAX],*pstr,*lptr,tline[128];
   char linkpath[NAME_MAX],*parent_dir;
   struct tm *gm_time, *cur_time;
   time_t  now;
   struct group *ginfo;
   struct passwd *uinfo;
   register int i;

   ret = SEND_PAGE;

   memset(&m,0x0,sizeof(m));
   sret=stat(pathname,&m);

   if ( sret < 0 ) {
       ret=setup_error(RC_INTERNAL_SERVER_ERROR,
                   "Unable to get stat info on  %s : %s",
                   pathname,strerror(errno));
       return ret;
   }

   now = time (NULL);
   cur_time = gmtime(&now);

   if ( S_ISDIR(m.st_mode) ){

       dir = opendir(pathname);

       if ( dir != NULL ) {

           start_page();
           start_body();
           start_content();
 
           start_div(page_config.max_textarea_cols*
                     page_config.char_pixel_width,
                     page_config.max_textarea_rows*
                     page_config.char_pixel_height);

           start_table(1,0,0,0,page_config.table_bg_color,0);

           while (  (dirent=readdir(dir)) != NULL ) {

           //switch( dirent->d_type ) {
           //    case DT_UNKNOWN:

               memset(&m,0x0,sizeof(m));
               memset(fullname,0x0,sizeof(fullname));
               if (  strcmp(dirent->d_name,"..") == 0 ) {
                   parent_dir=dirname(strdup(pathname));
                   sprintf(fullname,"%s",parent_dir);
                   free(parent_dir);
               } else {
                   if (  strcmp(dirent->d_name,".") == 0 ) {
                       sprintf(fullname,"%s",pathname);
                   } else {
                       //if (  strcmp(dirent->d_name,"/") == 0 ) {
                       if (  strcmp(pathname,"/") == 0 ) {
                           sprintf(fullname,"%s%s",pathname,dirent->d_name);
                       } else {
                           sprintf(fullname,"%s%c%s",pathname,path_sep,dirent->d_name);
                       }
                   }
               }

               sret=stat(fullname,&m);

               if ( sret != -1 ) {

                   start_row();

                   pstr = get_perm_str(&m);

                   gm_time = gmtime(&m.st_mtime);

                   start_col_new("top","",page_config.table_col_width,-1,-1,"");
                   append_page_data_body("%s",pstr);
                   end_col();

                   start_col_new("top","",page_config.table_col_width,-1,-1,"");
                   append_page_data_body("%-2d",m.st_nlink);
                   end_col();
 
                   uinfo=getpwuid(m.st_uid);
                   if ( uinfo ) {
                       start_col_new("top","",page_config.table_col_width,-1,-1,"");
                       append_page_data_body("%s",uinfo->pw_name);
                       end_col();
                   } else {
                       start_col_new("top","",page_config.table_col_width,-1,-1,"");
                       append_page_data_body("%d",m.st_uid);
                       end_col();
                   }

                   ginfo = getgrgid(m.st_gid);
                   if ( ginfo ) {
                       start_col_new("top","",page_config.table_col_width,-1,-1,"");
                       append_page_data_body("%s",ginfo->gr_name);
                       end_col();
                   } else {
                       start_col_new("top","",page_config.table_col_width,-1,-1,"");
                       append_page_data_body("%d",m.st_gid);
                       end_col();
                   }

                   start_col_new("top","",page_config.table_col_width,-1,-1,"");
                   append_page_data_body("%-12ld", m.st_size);
                   end_col();

                   memset(tline,0x0,sizeof(tline));
                   strftime(tline,sizeof(tline),"%b %d %Y",gm_time);
                   start_col_new("top","",page_config.table_col_width,-1,-1,"");
                   append_page_data_body("%s",tline);
                   end_col();

                   memset(tline,0x0,sizeof(tline));
                   if ( cur_time->tm_year > gm_time->tm_year ) {
                       strftime(tline,sizeof(tline),"%Y",gm_time);
                   } else {
                       strftime(tline,sizeof(tline),"%H:%M:%S",gm_time);
                   }
                   start_col_new("top","",page_config.table_col_width,-1,-1,"");
                   append_page_data_body("%s",tline);
                   end_col();

                   start_col_new("top","",page_config.table_col_width,-1,-1,"");
                   create_dir_link(pathname,fullname,dirent->d_name,&m);
                   end_row_col();
               }
               //break;
               //case DT_FIFO:
               //break;
               //case DT_CHR:
               //break;
               //case DT_DIR:
               //break;
               //case DT_BLK:
               //break;
               //case DT_REG:
               //break;
               //}
           }
           end_table();
           end_div();
           end_content();
           end_body();
           end_page();

           closedir(dir);

      } else {
          ret=setup_error(RC_NOT_FOUND,
             "Unable to open directory %s: %s",pathname,strerror(errno));
      }

  } else {
      ret=setup_error(RC_INTERNAL_SERVER_ERROR,
                      "Cannot dirplay %s : %s",pathname,strerror(errno));
  }

  return ret;

}

int create_dir_link(char *pathname,char *fullname,char *name,struct stat *m )
{
   char linkpath[NAME_MAX],tmpname[128];
   struct stat sbuf;
   int ret, sret;
   char *dirc,*basec,*dname,*bname;

   memset(linkpath,0x0,sizeof(linkpath));
   memset(tmpname,0x0,sizeof(tmpname));
   dirc=basec=dname=bname=NULL;

   switch(m->st_mode & __S_IFMT) {
       case __S_IFDIR:
           if ( (strcasecmp(name,"..") == 0 || strcasecmp(name,".") == 0 ) && 
               strcasecmp(pathname,"/") == 0 ) {
               start_bold();
               add_dir_str(name);
               end_bold();
           } else {
               add_dir_link(fullname,name,name);
           }
       break;
       case __S_IFLNK:
           memset(linkpath,0x0,sizeof(linkpath));
           ret=readlink(fullname,linkpath,sizeof(linkpath));
           if ( ret > 0 ) {
               if ( (sret=stat(linkpath,&sbuf)) != -1 ) {
                   dirc = strdup(linkpath);
                   basec = strdup(linkpath);
                   dname = dirname(dirc);
                   bname = basename(basec);
                   // Recursion will stop when we reach a non link
                   create_dir_link(dname,linkpath,bname,&sbuf);
                   free(dirc);
                   free(basec);
               }
           } else {
               add_dir_link(fullname,name,name);
               //add_dir_link(pathname,name,m,name);
           }
       break;
       case __S_IFREG:
           add_file_link(fullname,name,name);
           //add_file_link(pathname,name,m,name);
       break;
       case __S_IFMT:
           add_dir_link(fullname,name,name);
           //add_dir_link(pathname,name,m,name);
       break;
       case __S_IFIFO:
           add_dir_link(fullname,name,name);
           //add_dir_link(pathname,name,m,name);
       break;
       case __S_IFCHR:
           add_dir_link(fullname,name,name);
           //add_dir_link(pathname,name,m,name);
       break;
       case __S_IFBLK:
           add_dir_link(fullname,name,name);
           //add_dir_link(pathname,name,m,name);
       break;
       //case __S_IFMPC:
       //    add_dir_link(fullname,name,name);
       //    add_dir_link(pathname,name);
       //break;
       //    add_dir_link(fullname,name,name);
       //    add_dir_link(pathname,name);
       //case __S_IFMPB:
       //break;
       case __S_IFSOCK:
           add_dir_link(fullname,name,name);
           //add_dir_link(pathname,name,m,name);
       break;
       default:
           add_dir_link(fullname,name,name);
           //add_dir_link(pathname,name,m,name);
       break;
      
    }

}

page_status_type copyfile(char *from, char **newname )
{
   struct stat sbuf;
   page_status_type ret;
   FILE *fptr, *tptr;
   char *buf;
   int from_fd,to_fd,fret;
   long offset=0, len=0,cnt=0;
   char to[PATH_MAX],*newfile,*cwd;

   ret=DONT_SEND;

   cwd=session_get_cwd();

   newfile=find_params_value(EDIT_PAGE_PARAM_SAVEASFILE);
   if ( newfile  == NULL ) {
       ret=setup_error(RC_INTERNAL_SERVER_ERROR,"Parameter %s not found",
                                     "saveasfile");
   }
   memset(to,0x0,sizeof(to));
   sprintf(to,"%s/%s",strcasecmp(cwd,"/")==0?"":cwd,newfile);
   free(newfile);
   fret=stat(from,&sbuf);
   if ( fret < 0 ) {
       ret=setup_error(RC_INTERNAL_SERVER_ERROR,"Stat %s failed: %s",
          from,strerror(errno));
   }
   from_fd = open(from,O_RDONLY);
   if ( from_fd < 0 ) {
       ret=setup_error(RC_INTERNAL_SERVER_ERROR,"Open %s failed: %s",
          from,strerror(errno));
   }
   to_fd =open(to,O_WRONLY|O_CREAT|O_TRUNC);
   if ( to_fd < 0 ) {
       ret=setup_error(RC_INTERNAL_SERVER_ERROR,"Open %s failed: %s",
          to,strerror(errno));
   } else {
       offset=0;
       len=sbuf.st_size;
       do {
           cnt=sendfile(to_fd,from_fd,&offset,len);
           len -= cnt;
       } while( len > 0 );
       close(to_fd);
   }

   free_string(newname);
   *newname = strdup(to);

   close(from_fd);
   return ret;
}

page_status_type renamefile(char *from, char **newname )
{
   page_status_type ret;
   char newpath[PATH_MAX],*renamefile,*cwd;
   int fret;

   ret = DONT_SEND;
   cwd=session_get_cwd();
   renamefile=find_params_value(EDIT_PAGE_PARAM_RENAMEFILE);
   if ( renamefile  == NULL ) {
       ret=setup_error(RC_INTERNAL_SERVER_ERROR,"Parameter %s not found",
                                     EDIT_PAGE_PARAM_RENAMEFILE);
   }
   memset(newpath,0x0,sizeof(newpath));
   sprintf(newpath,"%s/%s",cwd,renamefile);
   fret=rename(from,newpath);
   if ( fret < 0 ) {
       ret=setup_error(RC_INTERNAL_SERVER_ERROR,"Rename %s failed: %s",
                       from,strerror(errno));
   }
   free_string(newname);
   *newname=renamefile;
   return ret;
}

page_status_type removefile( char *pathname )
{
    page_status_type ret;
    int fret;
    ret = DONT_SEND;
    fret=remove(pathname);
    if ( fret < 0 ) {
        ret=setup_error(RC_INTERNAL_SERVER_ERROR,
                        "Delete %s failed: %s",pathname,strerror(errno));
    }
    return ret;
}


int is_zero_read(char *pathname)
{
    int fd,rret,ret;
    struct stat m;
    char buf[1];
    memset(&m,0x0,sizeof(m));
    ret=stat(pathname,&m);
    if ( ret == 0 ) {
        if ( S_ISREG(m.st_mode) && m.st_size==0){
            // Check to see if stat is lying to us like in /proc zero byte file
            memset(buf,0x0,sizeof(buf));
            fd=open(pathname,O_RDONLY);
            if ( fd > 0 ) {
                rret=read(fd,buf,sizeof(buf));
                close(fd);
            }
            if ( rret >  0 ) {
               //stat lied; its size is zero; it has readable content
               return 1;
            }
        }
    }
    return 0;
}

