/*
 * Copyright (c) 2005 Kirk Murrin, <mailto:linuxguru1968@yahoo.com>
 * Licensed under the Gnu Public License, version 2.
 * All rights resered.
 * 
 */

#include <string.h>
#include <syslog.h>
#include <sys/wait.h>
#include <ctype.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>


#include "help.h"
#include "mconfig.h"
#include "html.h"
#include "pages.h"
#include "response.h"

extern int errno;
extern html_config_type page_config;
extern sys_config_type sys_config;

help_type *Help_System=NULL;

char *get_help_line( void )
{
    register int i; 
    char ch;
    static char line[128];
    memset(line,0x0,sizeof(line));
    i=0;
    while ( (ch=fgetc(Help_System->fptr)) != EOF ) {
        if ( i >= sizeof(line) ) {
           return NULL;
        }
        if ( ch == '\n' ) {
           line[i]='\0';
           break;
        }
        line[i]=ch;
        i++;
    }
    if ( ch == EOF ) 
        return NULL;
    else
        return line;
}

int init_help_system ( void )
{
   char *line=NULL;
   long length,start,end;
   char *desc=NULL, *var=NULL;

   length=start=end=0;
   close_help_system();

   Help_System=(help_type*)malloc(sizeof(help_type));
   if ( Help_System == NULL ) {
       syslog(LOG_ERR|LOG_USER,"init_help_system: malloc failed: %s",
             strerror(errno));
       return -1;
   }
   memset(Help_System,0x0,sizeof(help_type));
   Help_System->help_file_path=strdup(sys_config.help_file_path);

   //Help_System->fptr=fopen(page_config.help_file_path,"r");
   Help_System->fptr=fopen(Help_System->help_file_path,"r+");
   if ( Help_System->fptr == NULL ) {
       syslog(LOG_ERR|LOG_USER,"init_help_system: problem opening %s: %s",
             Help_System->help_file_path,strerror(errno));
       return -1;
   }

   Help_System->start_comment=ftell(Help_System->fptr);
   while ( (line=get_help_line())!=NULL && line[0] == '#' ) {
        //printf("Comment:\"%s\"\n",line);
   }

   Help_System->comment_length=ftell(Help_System->fptr) - 
                                Help_System->start_comment;

   line=get_help_line();
   if ( line == NULL ) return -1;
   desc = strdup(line);

   while( !feof(Help_System->fptr) ) {
        line=get_help_line();
        if ( line == NULL )  {
             var = NULL;
             start = ftell(Help_System->fptr);
             end=ftell(Help_System->fptr);
             goto add_entry;
        }
        if ( isspace(line[0]) ){
            var = desc;
            desc = NULL;
            goto start_body;
        } else {
            var = strdup(line);
        }
        start = ftell(Help_System->fptr);
        while( (line=get_help_line()) != NULL  )
        {
            if ( line == NULL ) break;
start_body:
            if ( line[0] == '#' ) continue;
            if ( line[0] == '\0' || isspace(line[0]) ){
                end=ftell(Help_System->fptr);
                continue;
            } else {
add_entry:
                length=end-start;
                //printf("Desc: %s\n",desc);
                //printf("Var: %s\n",var); 
                //printf("Offset: %ld\n", start);
                //printf("Length: %ld\n",length);
                if ( (Help_System->entries=(help_entry_type*)realloc(Help_System->entries,
                     (Help_System->numentries+1)*sizeof(help_entry_type))) == NULL ) {
                    syslog(LOG_ERR|LOG_USER,"init_help_system:realloc failed: %s",
                           strerror(errno));
                    return -1;
                }
         
                Help_System->entries[Help_System->numentries].description=(desc==NULL?strdup(""):desc);
                desc = NULL;
                memset(Help_System->entries[Help_System->numentries].variable,0x0,
                     sizeof(Help_System->entries[Help_System->numentries].variable));
                memcpy(Help_System->entries[Help_System->numentries].variable,var,strlen(var));
                free(var); var=NULL;
                Help_System->entries[Help_System->numentries].offset=start;
                Help_System->entries[Help_System->numentries].length=length;
                Help_System->numentries++;

                // Starting new block
                desc = strdup(line);
                break;
            }
        }
   } 
   if ( var ) free(var); var=NULL;
   qsort(Help_System->entries,Help_System->numentries,sizeof(help_entry_type),compare_help_var);
   rewind(Help_System->fptr);
   output_help_file();

   return 0;

}


int compare_help_var(const void *s1, const void *s2)
{
    return strcmp(((help_entry_type*)s1)->variable,((help_entry_type*)s2)->variable);
}

int compare_help_desc(const void *s1, const void *s2)
{
    return strcmp(((help_entry_type*)s1)->description,((help_entry_type*)s2)->description);
}

void close_help_system( void )
{
    register long i;

    if ( Help_System != NULL ) {
        for(i=0; i < Help_System->numentries; i++) {
            if (  &Help_System->entries[i] ) {
                memset(Help_System->entries[i].variable,0x0,
                       sizeof(Help_System->entries[i].variable));
                if ( Help_System->entries[i].description != NULL ){
                    free(Help_System->entries[i].description);
                    Help_System->entries[i].description=NULL;
                }
                Help_System->entries[i].offset=0;
                Help_System->entries[i].length=0;
            }
        }
        if ( Help_System->help_file_path ) {
             free(Help_System->help_file_path);
             Help_System->help_file_path=NULL;
        }
        if ( Help_System->entries) {
            free(Help_System->entries);
            Help_System->entries=NULL;
        }
        Help_System->numentries=0;

        if ( Help_System -> fptr != NULL ) { 
            fclose(Help_System -> fptr);
            Help_System->fptr=NULL;
        }
        free(Help_System);
        Help_System=NULL;
    }
}

void print_help( void )
{
    register int i;
    for(i=0; i < Help_System->numentries; i++) {
        if ( &Help_System->entries[i] != NULL ) {
        printf("%d: %s %s %ld %ld\n",i,
            Help_System->entries[i].variable,
            Help_System->entries[i].description,
            Help_System->entries[i].offset,
            Help_System->entries[i].length);
        }
    }
}

/*
void build_help_file( void )
{
    int pid, status;
    pid=output_help_file();
    waitpid(pid,&status,0);
}
*/

int output_help_file( void )
{
   int pid,child_exit_status,ws;
   register int i;
   FILE *fp;

   if ( (pid=fork()) == 0 ) {
       fp = fopen(sys_config.help_path,"w+");
       append_file_data(fp,"<html>\n<title></title>\n<body>\n");
       //FIXME: put this table params in page config
       append_file_data(fp,"<table cellSpacing=0 cellPadding=0 width=100%% height=0%%   bgColor=\"grey\" border=0\n");
       qsort(Help_System->entries,Help_System->numentries,sizeof(help_entry_type),compare_help_desc);
       for( i =0 ; i < Help_System->numentries; i++ ) {
           if ( &Help_System->entries[i] != NULL ) {
               append_file_data(fp,"<tr><td><a href=\"%s?%s=%s\" target=_parent>%s</a></td></tr>\n",
                   HELP_URL,PARAM_TARGET,Help_System->entries[i].variable,
                   strlen(Help_System->entries[i].description) == 0?
                   Help_System->entries[i].variable:
                   Help_System->entries[i].description);
                   fflush(fp);
                   fsync(fileno(fp));
           }
       }
       append_file_data(fp,"</table></body>\n</html>\n");
       fclose(fp);
       qsort(Help_System->entries,Help_System->numentries,sizeof(help_entry_type),compare_help_var);
       _exit(pid);
    }
    child_exit_status=0;
    ws = waitpid(pid,&child_exit_status,0);
    if( WIFEXITED(child_exit_status) ) {
      syslog(LOG_ERR|LOG_USER,"output_help_file: chiled extied normally");
    } else {
      syslog(LOG_ERR|LOG_USER,"output_help_file: chiled extied abnormally");
    }
    return pid;
}

help_type *get_help_system( void )
{
    return Help_System;
}
