/*
 * Copyright (c) 2005 Kirk Murrin, <mailto:linuxguru1968@yahoo.com>
 * Licensed under the Gnu Public License, version 2.
 * All rights resered.
 * 
 */

#include <syslog.h>

#include "mconfig.h"
#include "help.h"
#include "response.h"
#include "html.h"
#include "request.h"
#include "page_util.h"

#include "help_page.h"

extern int errno;
extern html_config_type page_config;
extern sys_config_type sys_config;

#define HELP_PAGE_ACTION_REFRESH "Refresh"
#define HELP_PAGE_ACTION_RELOAD  "Reload"

page_type help_page = {
        "Help",
        "&nbsp;",
        "&nbsp;",
        handle_help, 
        {
             INPUT,   { "submit",    PARAM_ACTION, HELP_PAGE_ACTION_REFRESH,   },
             INPUT,   { "submit",    PARAM_ACTION, HELP_PAGE_ACTION_RELOAD,    },
             NONE,{ "",          "",          "",           }
        }
};

int handle_help( void )
{
   int pid,status;
   char *method,*action;
   action = get_request_action();
   method = get_request_method();

   if ( strcasecmp(method,"GET") == 0 ) {
   } else {
       if ( strcasecmp(method,"POST") == 0 ) {
           if ( strcmp(action,HELP_PAGE_ACTION_REFRESH) == 0 ) {
           } else {
               if ( strcmp(action,HELP_PAGE_ACTION_RELOAD) == 0 ) {
                   init_help_system();
               }
           }
       }
   }
   return setup_help();
}


int setup_help( void )
{
    char *buf, *config_var,*target;
    help_entry_type *hp;
    int ret=0,len,rc;
    register int i;
    int fd;
    long offset,length;
    help_type *help_system;
    response_type *response;

    help_system = get_help_system();

    if ( help_system == NULL || help_system->fptr == NULL ||
         help_system->entries == NULL ){
        setup_error(RC_INTERNAL_SERVER_ERROR,"Help System not initialized");
        return SEND_PAGE;
    }

    set_response_status_code(RC_OK);
    send_status();
    send_server();
    send_date();
    send_connection();
    send_content_type();
    send_expires();
    send_cache_control();
    send_pragma();
    send_line("Transfer-Encoding: chunked\r\n");
    send_blank_line();

    target = get_request_target();
    response = get_response();

    if ( target != NULL ) {
        config_var = strdup(target); 
        hp=bsearch(config_var,help_system->entries,help_system->numentries,sizeof(help_entry_type),compare_help_var);
        if ( hp != NULL ) {
            set_response_title("%s [%s]",config_var,hp->description);

            clear_response_data_head();
            start_page();
            start_body();
            send_line("%X;\r\n",response->head.datalen);
            rc = send_data(&response->head.data,response->head.datalen);

            clear_response_data_body();
            start_content();
            start_textarea( 
                  page_config.max_textarea_rows,
                  page_config.max_textarea_cols,
                  1,sys_config.build_path);

            append_page_data_body("\n  %s [%s]\n\n",config_var,hp->description);
            send_line("%X\r\n",response->body.datalen);
            rc = send_data(&response->body.data,response->body.datalen);

            rewind(help_system->fptr);
            offset=hp->offset;
            length=hp->length;
            fd=fileno(help_system->fptr);
            send_line("%X\r\n",length);
            rc=sendfile_data(fd,&offset,length);
            //FIXME: inline error handling??
            if ( rc < 0 ) {
               //syslog(LOG_ERR|LOG_USER,"setup_help: sendfile failed %s",strerror(errno));
               //setup_error(RC_INTERNAL_SERVER_ERROR,"setup_help; senfile failed %s",strerror(errno));
               //return SEND_PAGE;
            }

            clear_response_data_body();
            end_textarea();
            end_content();
            send_line("%X\r\n",response->body.datalen);
            rc = send_data(&response->body.data,response->body.datalen);

            clear_response_data_tail();
            end_body();
            end_page();
            send_line("%X\r\n",response->tail.datalen);
            rc = send_data(&response->tail.data,response->tail.datalen);
            free(config_var);

        } else {
            setup_error(RC_NOT_FOUND,"Cannot find help on: %s",config_var);
            free(config_var);
            return SEND_PAGE;
        }

    } else { 

        set_response_title(NULL);
        clear_response_data_head();
        start_page();
        start_body();
        send_line("%X;\r\n",response->head.datalen);
        rc = send_data(&response->head.data,response->head.datalen);

        clear_response_data_body();
        start_content();
        send_line("%X\r\n",response->body.datalen);
        rc=send_data(&response->body.data,response->body.datalen);

        clear_response_data_body();
        start_iframe(sys_config.help_path,sys_config.help_path, 
               page_config.max_textarea_cols*
               page_config.char_pixel_width,
               page_config.max_textarea_rows*
               page_config.char_pixel_height,0);
         append_page_data_body("If you can see this, your browser doesn't understand IFRAME. \
However, we'll still <a href=\"%c%s\" type=\"text/plain\">link</a> you to the file. \n",path_sep,sys_config.help_path); 
        end_iframe();
        send_line("%X\r\n",response->body.datalen);
        rc=send_data(&response->body.data,response->body.datalen);

        //start_div(page_config.max_textarea_cols*
        //          page_config.char_pixel_width,
       //           page_config.max_textarea_rows*
        //          page_config.char_pixel_height);

        //start_table(1,0,0,0,page_config.table_bg_color,0);
        //send_line("%X\r\n",Response.body.datalen);
        //rc = send(sd, Response.body.data,Response.body.datalen,0);
        //fsync(sd);
        // TODO: Build in memory in init or dump to file iframe ??
        //qsort(help_system->entries,help_system->numentries,sizeof(help_entry_type),compare_help_desc);
        //for( i =0 ; i < help_system->numentries; i++ ) {
        //    if ( &help_system->entries[i] != NULL ) {
        //        clear_response_data_body();
        //        start_row_col_new(page_config.table_row_height,page_config.table_col_width);
        //        append_page_data_body("<a href=\"/main/help/%s\" target=_parent>%s</a>\n",
        //            help_system->entries[i].variable,
        //            strlen(help_system->entries[i].description) == 0?
        //            help_system->entries[i].variable:
        //            help_system->entries[i].description);
        //        end_row_col();
        //        send_line("%X\r\n",Response.body.datalen);
        //        rc = send(sd,Response.body.data,Response.body.datalen,0);
        //    }
        //}
        //qsort(help_system->entries,help_system->numentries,sizeof(help_entry_type),compare_help_var);
        //end_table();
        //end_div();

        clear_response_data_body();
        end_content();
        send_line("%X\r\n",response->body.datalen);
        rc = send_data(&response->body.data,response->body.datalen);

        clear_response_data_tail();
        end_body();
        end_page();
        send_line("%X\r\n",response->tail.datalen);
        rc = send_data(&response->tail.data,response->tail.datalen);
        fsync_data();

    }

    send_line("0\r\n");
    send_blank_line();

    fsync_data();

    return DONT_SEND;

}

/*
int handle_help( void )
{
   int pid,status;
   char *method,*action;

   method = get_request_method();
   action = get_request_action();

   if ( strcasecmp(method,"GET") == 0 ) {
   } else {
       if ( strcasecmp(method,"POST") == 0 ) {
           if ( strcmp(action,"Refresh") == 0 ) {
           } else {
               if ( strcmp(action,"Reload") == 0 ) {
                   init_help_system();
               }
           }
       }
   }
   return setup_help();
}

int setup_help( void )
{
    char *buf, *config_var;
    help_entry_type *hp;
    int ret=0,len,rc,fd;
    register int i;
    long offset,length;
    help_type *help_system;
    char *uri;
    page_data_type *head,*body,*tail;

    clear_response();

    //uri = get_request_uri();
    uri = get_request_target();

    head = get_response_head();
    body = get_response_body();
    tail = get_response_tail();

    help_system = get_help_system();

    if ( help_system == NULL || help_system->fptr == NULL ||
         help_system->entries == NULL ){
        setup_error(RC_INTERNAL_SERVER_ERROR,"Help System not initialized");
        return SEND_PAGE;
    }

    send_status(RC_OK);
    send_date();
    send_server();
    //send_line("Server: %s %s\r\n",argument.cmdname,argument.version);
    send_connection();
    send_content_type();
    //send_line("Content-Type: %s\r\n",get_response_content_type());
    send_expires();
    //send_line("Expires: 300\r\n");
    send_cache_control();
    //send_line("Cache-Control: private\r\n");
    send_pragma();
    //send_line("Pragma: no-cache\r\n");
    send_line("Transfer-Encoding: chunked\r\n\r\n");

    if ( strlen(uri) > 0 ) {
        config_var = strdup(uri); 
        hp=bsearch(config_var,help_system->entries,help_system->numentries,sizeof(help_entry_type),compare_help_var);
        if ( hp != NULL ) {

            set_response_title("%s [%s]",config_var,hp->description);
            clear_response_data_head();
            start_page();
            start_body();
            send_line("%X;\r\n",head->datalen);
            rc = send_data(&head->data,head->datalen);

            clear_response_data_body();
            start_content();

            start_textarea( 
                  page_config.max_textarea_rows,
                  page_config.max_textarea_cols,
                  1,sys_config.build_path);

            append_page_data_body("\n  %s [%s]\n\n",config_var,hp->description);

            send_line("%X\r\n",body->datalen);
            rc = send_data(&body->data,body->datalen);

            rewind(help_system->fptr);
            offset=hp->offset;
            length=hp->length;
            fd=fileno(help_system->fptr);
            send_line("%X\r\n",length);
            rc=sendfile_data(fd,&offset,length);
            if ( rc < 0 ) {
               syslog(LOG_ERR|LOG_USER,"setup_help: sendfile failed %s",strerror(errno));
               setup_error(RC_INTERNAL_SERVER_ERROR,"setup_help; senfile failed %s",strerror(errno));
               return SEND_PAGE;
            }

            clear_response_data_body();
            end_textarea();
            end_content();

            send_line("%X\r\n",body->datalen);
            rc = send_data(&body->data,body->datalen);

            clear_response_data_tail();
            end_body();
            end_page();

            send_line("%X\r\n",tail->datalen);
            rc = send_data(&tail->data,tail->datalen);

            free(config_var);

        } else {
            setup_error(RC_NOT_FOUND,"Cannot find help on: %s",config_var);
            free(config_var);
            return SEND_PAGE;
        }

    } else { 

        set_response_title(NULL);
        clear_response_data_head();
        start_page();
        start_body();

        send_line("%X;\r\n",head->datalen);
        rc = send_data(&head->data,head->datalen);

        clear_response_data_body();
        start_content();

        send_line("%X\r\n",body->datalen);
        rc=send_data(&body->data,body->datalen);

        clear_response_data_body();
        start_iframe(sys_config.help_path,sys_config.help_path, 
               page_config.max_textarea_cols*
               page_config.char_pixel_width,
               page_config.max_textarea_rows*
               page_config.char_pixel_height,0);
         append_page_data_body("If you can see this, your browser doesn't understand IFRAME. \
However, we'll still <a href=\"%c%s\" type=\"text/plain\">link</a> you to the file. \n",path_sep,sys_config.help_path); 
        end_iframe();

        send_line("%X\r\n",body->datalen);
        rc=send_data(&body->data,body->datalen);

        //
        //start_div(page_config.max_textarea_cols*
        //          page_config.char_pixel_width,
        //          page_config.max_textarea_rows*
        //          page_config.char_pixel_height);

        //start_table(1,0,0,0,page_config.table_bg_color,0);

        //send_line("%X\r\n",body->datalen);
        //rc = send_data(&body->data,body->datalen);
        //fsync(sd);


        // TODO: Build in memory in init or dump to file iframe ??
        //qsort(help_system->entries,help_system->numentries,sizeof(help_entry_type),compare_help_desc);
        //for( i =0 ; i < help_system->numentries; i++ ) {
        //    if ( &help_system->entries[i] != NULL ) {
        //        clear_response_data_body();
        //        start_row_col_new(page_config.table_row_height,page_config.table_col_width);
        //        append_page_data_body("<a href=\"/main/help/%s\" target=_parent>%s</a>\n",
        //            help_system->entries[i].variable,
        //            strlen(help_system->entries[i].description) == 0?
        //            help_system->entries[i].variable:
        //            help_system->entries[i].description);
        //        end_row_col();
        //        send_line("%X\r\n",body->datalen);
        //        rc = send_data(&body->data,body->datalen);
         //   }
        //}
        //qsort(help_system->entries,help_system->numentries,sizeof(help_entry_type),compare_help_var);
        //end_table();
        //end_div();
        //

        clear_response_data_body();
        end_content();
        send_line("%X\r\n",body->datalen);
        rc = send_data(&body->data,body->datalen);

        clear_response_data_tail();
        end_body();
        end_page();
        send_line("%X\r\n",tail->datalen);
        rc = send_data(&tail->data,tail->datalen);

        fsync_data();

    }

    send_line("0\r\n\r\n");
    fsync_data();

    return DONT_SEND;

}
*/
