/*
 * Copyright (c) 2005 Kirk Murrin, <mailto:linuxguru1968@yahoo.com>
 * Licensed under the Gnu Public License, version 2.
 * All rights resered.
 * 
 */

// #define _GNU_SOURCE
//#define __USE_GNU
#include <string.h>
#include <stdlib.h>

#include "html.h"
#include "response.h"
#include "request.h"
#include "pages.h"
#include "page_util.h"

char *strndup(const char *s, size_t n);

// Configurable page attributes and prompts and labels
html_config_type page_config = {
        50,                      // Table width
        50,                      // Table height
        25,                      // textarea rows
        95,                      // textarea cols
        8,                       // char pixel width
        15,                      // char pixel height
        0,                       // button area cell spacing
        5,                       // button area cell padding
        150,                     //Top menu item width
        0,                       //Top menu item height
        180,                     //button_win_width;
        0,                       //button_win_height;
        100,                     //main_win_width;
        100,                     //main_win_height;
        10,                      //Table row height
        400,                     // Table col width
	"cyan",			 // page background color
        "grey",                  // table background color
        "grey",                  // Top menu bg color
        "grey",                  // Button area menu bg color
        //"logo.gif",		 // page background logo image
        //"peng.gif",		 // page background logo image
        "",		         // page background logo image
        0,			 // table border type
        "<?>",		         // Help prompt
        "-->"                    // Dir pointer
};

void start_page( void )
{
    //int  expires;
    //char *pragma,*content_type,*charset;
    //int  cache_control;
    doctype_type *doctype;

    doctype = get_response_doctype();

    append_page_data_head("<!DOCTYPE %s %s \"%s//%s//%s %s//%s\" \"%s\">\n",
                  doctype->top_element,
                  doctype->availability,
                  doctype->registration,
                  doctype->organization,
                  doctype->type,
                  doctype->label,
                  doctype->language,
                  doctype->fpi);

    append_page_data_head("<html>\n<head>\n<title>%s</title>\n",get_response_title());

    //expires = get_response_expires();
    //append_page_data_head("<meta http-equiv=\"Expires\" CONTENT=\"%d\">\n",expires);

    //cache_control= get_response_cache_control();
    //append_page_data_head("<meta http-equiv=\"Cache-Control\" CONTENT=\"max-age=%d\">\n",cache_control);

    // For 1.0 brower
    //pragma = get_response_pragma();
    //append_page_data_head("<meta http-equiv=\"Pragma\" CONTENT=\"%s\">\n",pragma);

    //content_type= get_response_content_type();
    //charset     = get_response_charset();
    //append_page_data_head("<meta http-equiv=\"Content-Type\" content=\"%s; charset=%s\">\n",
    //        get_response_content_type(), get_response_charset());
}

void end_page( void )
{
  append_page_data_tail("</html>\n");
}


void start_body( void )
{
  append_page_data_head("<body bgcolor=\"%s\" ",page_config.page_bg_color);
  if ( strlen(page_config.page_bg_image) > 0 ) {
      append_page_data_head(" background=\"%c%s\" ",path_sep,page_config.page_bg_image);
  }
  append_page_data_head(">\n");
}

void end_body( void )
{
  append_page_data_tail("</body>\n");
}


void start_form(char *method )
{
  append_page_data_body("<form method=\"%s\">\n",method);
}

void end_form( void )
{
  append_page_data_body("</form>\n");
}


void add_caption( char *caption )
{
    if ( caption ) {
        append_page_data_body("<caption align=\"top\">%s</caption>\n",caption);
    }
}

void add_br(void)
{
  append_page_data_body("<br>\n");
}


void start_p( void )
{
    append_page_data_body("<p>\n");
}

void end_p( void )
{
    append_page_data_body("</p>\n");
}

void add_comment( char *comment )
{
  if ( comment ) {
      append_page_data_body("<!-- %s-->\n",comment);
  }
}


void start_table_percent(int cellSpacing, int cellPadding, int width, int height, char *bgcolor, int border)
{
    append_page_data_body("<TABLE cellSpacing=%d cellPadding=%d width=\"%d%%\" height=\"%d%%\"  bgColor=\"%s\" border=%d>\n",
           cellSpacing,cellPadding,width,height,bgcolor,border);
    //append_page_data_body("<TABLE>\n");
}

void start_table(int cellSpacing, int cellPadding, int width, int height, char *bgcolor, int border)
{
    append_page_data_body("<TABLE cellSpacing=%d cellPadding=%d width=%d height=%d bgColor=\"%s\" border=%d>\n",
           cellSpacing,cellPadding,width,height, bgcolor,border);
    //append_page_data_body("<TABLE>\n");
}

void end_table( void )
{
  append_page_data_body("</table>\n");
}


void start_div( int width, int height )
{
  //append_page_data_body("<DIV STYLE=\"overflow: auto; width: %d%; height: %d; border-left: 1px gray solid; border-bottom: 1px gray solid; padding:0px; margin: 0px\">\n", 
  append_page_data_body("<DIV STYLE=\"overflow: auto; width: %d; height: %d; border-left: 1px gray solid; border-bottom: 1px gray solid; padding:0px; margin: 0px\">\n", 
                   width,height);
}

void end_div( void )
{
  append_page_data_body("</DIV>\n");
}

/*
void start_tbody( void )
{
    append_page_data_body("<tbody>\n");
}


void end_tbody( void )
{
    append_page_data_body("</tbody>\n");
}
*/

void start_center( void ) 
{
    append_page_data_body("<center>\n");
}

void end_center( void ) 
{
    append_page_data_body("</center>\n");
}

void start_row( void ) 
{
    append_page_data_body("<tr>\n");
}

void start_row_new( int height ) 
{
    append_page_data_body("<tr height=%d>\n",height);
}

void end_row( void ) 
{
    append_page_data_body("</tr>\n");
}

void start_font( char *face, char *color , int size ) 
{
    append_page_data_body("<FONT face=\"%s\" color=\"%s\" size=%d>\n",
             face, color, size );
}

void end_font( void ) 
{
    append_page_data_body("</FONT>\n");
}

void start_col( void ) 
{
    append_page_data_body("<td align=&quot;left&quot; width=0 height=0>\n");
}


void start_col_color( char *color ) 
{
    append_page_data_body("<td bgcolor=\"%s\">\n",color);
}

void start_col_new(char *valign, char *align, int width, int height, int colspan, char *bgcolor)
{
    char buff[256];
    int len;
    len=0; 
    memset(buff,0x0,sizeof(buff));
    if ( strlen(valign) > 0 ) {
        len += sprintf(&buff[len]," valign=\"%s\"",valign);
    }
    if ( strlen(align) > 0 ) {
        len += sprintf(&buff[len]," align=\"%s\"",align);
    }
    if ( strlen(bgcolor) > 0 ) {
        len += sprintf(&buff[len]," bgcolor=\"%s\"",bgcolor);
    }

    if ( height >= 0 ) {
        len += sprintf(&buff[len]," height=%d",height);
    }

    if ( width >= 0 ) {
        len += sprintf(&buff[len]," width=%d",width);
    }
    if ( colspan >= 0 ) {
        len += sprintf(&buff[len]," colspan=%d",colspan);
    }
    append_page_data_body("<TD %s>\n",buff);
    //append_page_data_body(
    //          "<TD %s align=\"%s\" width=%d bgColor=\"%s\" colSpan=%d  height=%d>\n",
    //         "vAlign=\"%s\" align=\"%s\" width=%d bgColor=\"%s\" colSpan=%d  height=%d>\n",
    //        valign, align, width, bgcolor,  colspan, height);
}

void start_col_new_percent(char *valign, char *align, int width, int height, int colspan, char *bgcolor)
{
    append_page_data_body(
             "<TD vAlign=\"%s\" align=\"%s\" width=\"%d%\" height=\"%d%\" bgColor=\"%s\" colSpan=%d>\n",
             valign, align, width, height, bgcolor,  colspan);
}


void end_col( void ) 
{
    append_page_data_body("</td>\n");
}


void add_heading( char *heading ) 
{
    append_page_data_body("<th>%s</th>\n",heading );
}

void start_row_col( void ) 
{
    start_row();
    start_col();
}

void start_row_col_new( int height, int width ) 
{
    start_row_new(height);
    start_col_new("","",width,-1,-1,"");
}

void end_row_col( void )
{
    end_col();
    end_row();
}


void start_pointer_link(const statement_type *statement )
{
    piece_type  prompt = statement->sb_prompt->value;
    append_page_data_body(
                "<a href=\"%s?target=%8.8p\" title=\"%*.*s\" type=\"text/html\">",
                MAIN_CONF_URL,statement,prompt.len,prompt.len,prompt.ptr);
}

void start_link(char *href )
{
    append_page_data_body("<a href=\"%s\">",href);
}

void end_link( void )
{
    append_page_data_body("</a>\n");
}

void add_dir_link(char *dirpath, char *name, char *title )
{
    append_page_data_body("<a href=\"%s?target=%s\" title=\"%s\">&nbsp;%s&nbsp;%s</a>\n",
         MAIN_EDIT_DIR_URL,dirpath,title,name,page_config.dir_pointer_prompt);
}

void add_dir_str(char *name )
{
    append_page_data_body("&nbsp;%s&nbsp;%s\n",name,page_config.dir_pointer_prompt);
}

void add_file_link(char *filepath, char *name, char *title )
{
    append_page_data_body("<a href=\"%s?target=%s\" title=\"%s\">&nbsp;%s</a>\n",
          MAIN_EDIT_FILE_URL,filepath,title,name);
}

void add_help_link(symbol_type  *symbol)
{
  char *config_var, *cp;
  if ( symbol != NULL ){
          if ( symbol->name.ptr[0]  != '\0' ) {
              config_var = strndup(symbol->name.ptr,symbol->name.len);
              cp = strchr(config_var,'\n');
              if ( cp != NULL ) {
                 *cp = '\0';
              }
              append_page_data_body(
                        "<td><a href=\"%s?target=%s\" title=\"%s\">&lt;?&gt;</a></td>",
                               HELP_URL,config_var,config_var);
              free(config_var);
          }
      }
}


void add_submit(char *name, char *value)
{
    start_col();
    start_center();
    append_page_data_body(
             "<input type=\"submit\" name=\"%s\" bgcolor=\"cyan\" value=\"%s\"/>",
             name, value );
    end_center();
    end_col();
}

void start_textarea(int rows, int cols,  int readonly, char *name )
{
    append_page_data_body("<textarea rows=%d cols=%d %s name=\"%s\">\n",
             rows,cols,readonly?"readonly=\"readonly\"":"",name);
}

void end_textarea( void )
{
    append_page_data_body("</textarea>\n");
}


void start_frameset( void )
{
    append_page_data_body("<frameset rows=\"100,200\">\n");
}

void end_frameset( void )
{
    append_page_data_body("</frameset>\n");
}

void start_noframe( void )
{
    append_page_data_body("<noframes>\n");
}

void end_noframe( void )
{
    append_page_data_body("</noframes>\n");
}

void start_frame( char *src )
{
    append_page_data_body(
             "<frame SCROLLING=\"auto\" src=\"%s\">\n",src);
}

void end_frame( void )
{
    append_page_data_body("</frame>\n");
}


void start_iframe(char *name,char *src, int width, int height, int parent )
{
    append_page_data_body(
        "<iframe SCROLLING=\"auto\" src=\"%c%s\" width=%d height=%d >\n",
         path_sep,src,width, height);
}

void end_iframe( void )
{
    append_page_data_body("</iframe>\n");
}


void start_select(char *name, int size )
{
    append_page_data_body("<select name=\"%s\" size=%d>\n",name,size);
}


void end_select(char *nama, int size )
{
    append_page_data_body("</select>\n");
}

void add_option( char*value, char  *prompt )
{
    append_page_data_body("<option value=\"%s\">%s</option>\n",value,prompt);
}

void add_str(char *str)
{
    int len;
    if  ( str != NULL ) { 
        len = strlen(str);
        if ( len > 0 ) {
            append_page_data_body("%*.*s",len,len,str);
        }
    }
}

void start_bold( void )
{
    append_page_data_body("<b>\n");
}

void end_bold( void )
{
    append_page_data_body("</b>\n");
}

void start_pre( void )
{
    append_page_data_body("<pre>\n");
}

void end_pre( void )
{
    append_page_data_body("</pre>\n");
}

void start_content(void)
{
    int i;
    char *referer,*str,*token; 
    char *delims=" \n\t";
    int width, height;
    page_table_type *page_table_entry;
    page_type *curr_page;
    link_type *menu_ptr;
    page_table_type *page_ptr;
    char *menu_level;
 
    page_table_entry=get_request_curr_page();

    start_form("POST");
    start_center();
    add_br();
    start_table_percent(0,0,page_config.table_width,page_config.table_height,"",1);
    start_row();
    add_comment("MAIN TABLE");
    start_row();
    start_col_new("center","middle",0,0,4,page_config.top_menu_bg_color);
    start_table(0,0,0,0,"",0);
    start_row();

    menu_level = page_table_entry->url;

    sort_pages_menu_order();

    for( init_pages() ; more_pages() ; next_page() ) {

        page_ptr = get_page();

        //FIXME: Need func of macro for this
        if ( page_ptr->visible == DISPLAYED ) {

          /* start_col_new("center","middle",0,0,0,"");
          start_font("verdana","#efefef",2);
          append_page_data_body("|");
          end_font();
          end_col(); */
 
          start_col_new("center","middle",page_config.top_menu_item_width,
                                          page_config.top_menu_item_height,0,"");
          start_font("verdana","#000000",2);

          if ( strcasecmp(menu_level,ROOT_URL) != 0 ) {
              if ( strcasecmp(menu_level,page_ptr->url) == 0 ){
                  append_page_data_body("[<B>%s</B>]\n",
                               page_ptr->page->title);
              } else {
                  append_page_data_body("[<A href=\"%s\">%s</A>]\n",
                              page_ptr->url,
                              page_ptr->page->title);
              }
          } else {
              if ( strcasecmp(page_ptr->url,MAIN_CONF_URL) == 0 ) {
                  append_page_data_body("[<B>%s</B>]\n",
                           page_ptr->page->title);
              } else {
                  append_page_data_body("[<A href=\"%s\">%s</A>]\n",
                           page_ptr->url,
                           page_ptr->page->title);
              }
          }

          end_font();
          end_col();
          // add padding between menu items 
          //start_col_new_percent("center","middle",page_config.menu_width,page_config.menu_height,0,"");
          //append_page_data(&Response.body,"&nbsp;"); end_col();

        }
    }

    //put pages back in search order
    sort_pages_page_order();

    end_row();
    end_table();

    end_col();
    end_row();

    curr_page = page_table_entry->page;

    start_row();
    start_col_new("center","middle",0,0,4,"#efefef");
    append_page_data_body("%s\n",curr_page->header);
    end_col();
    end_row();
    start_row();
    start_col_new("top","left",page_config.button_win_width,page_config.button_win_height,0,page_config.button_area_bg_color);
    start_font("verdana","#000000",2);

    start_table(page_config.button_area_cell_spacing,
                        page_config.button_area_cell_padding,
                        page_config.button_win_width,
                        page_config.button_win_height,"",0);

    for(i=0; curr_page->tags[i].type != NONE;i++ ) {
        append_page_data_body("<tr><td align=\"left\" valign=\"center\">\n");
        if ( curr_page->tags[i].type == INPUT ) {
            append_page_data_body("&nbsp;\n");
            if ( strcmp("text",curr_page->tags[i].tag.input.type) == 0 ) {
                append_page_data_body("<input type=\"%s\" size=\"%s\" maxlength=%d name=\"%s\" bgcolor=\"cyan\" value=\"%s\"></input>\n",
                    curr_page->tags[i].tag.input.type, 
                    curr_page->tags[i].tag.input.size,
                    curr_page->tags[i].tag.input.maxlen,
                    curr_page->tags[i].tag.input.name,
                    curr_page->tags[i].tag.input.value);

            } else {
                if ( strcmp("checkbox",curr_page->tags[i].tag.input.type) == 0 ) {
                    append_page_data_body("<input type=\"%s\" name=\"%s\" bgcolor=\"cyan\" value=\"%s\" size=\"%s\" maxlength=%d %s>&nbsp;%s</input>\n",
                        curr_page->tags[i].tag.input.type, 
                        curr_page->tags[i].tag.input.name,
                        curr_page->tags[i].tag.input.value,
                        curr_page->tags[i].tag.input.size,
                        curr_page->tags[i].tag.input.maxlen,
                        ((curr_page->tags[i].tag.input.checked>0)?"checked":""),
                        curr_page->tags[i].tag.input.value
                        );

                } else {
                    append_page_data_body("<input type=\"%s\" name=\"%s\" bgcolor=\"cyan\" value=\"%s\" size=\"%s\" maxlength=%d></input>\n",
                        curr_page->tags[i].tag.input.type, 
                        curr_page->tags[i].tag.input.name,
                        curr_page->tags[i].tag.input.value,
                        curr_page->tags[i].tag.input.size,
                        curr_page->tags[i].tag.input.maxlen);
                }
            }
        } else {
            if ( curr_page->tags[i].type == SELECT ) {
                append_page_data_body("<select name=\"%s\" size=%d>\n",
                    curr_page->tags[i].tag.select.name, 
                    curr_page->tags[i].tag.select.size );

                str = strdup(curr_page->tags[i].tag.select.options);
                if ( str ){
                    token=strtok(str,delims);
                    if ( token != NULL ) {
                        append_page_data_body("<option value=\"%s\">%s</option>\n",token,token);
                        while( (token=strtok(NULL,delims)) != NULL ){
                            append_page_data_body("<option value=\"%s\">%s</option>\n",token,token);
                        }
                    }
                }
                append_page_data_body("</select>\n");
                free(str);
            } else {
                if ( curr_page->tags[i].type == LABEL ) {
                    append_page_data_body("%s\n",
                           curr_page->tags[i].tag.label.text);

                }
            }
        }
        append_page_data_body("</td></tr>\n");
    }
    end_table();


    end_p();
    end_font();
    end_col();
    start_col_new_percent("top","left",page_config.main_win_width,page_config.main_win_height,0,
                          page_config.table_bg_color);
    add_comment("Begin Content");
}

void end_content(void)
{
    page_type *curr_page;

    curr_page = get_request_curr_page()->page;

    add_comment("End Content");
    end_col();
    start_row();
    start_col_new_percent("center","middle",page_config.table_width-4,1,5,"#a4a4a4");
    append_page_data_body("%s\n",curr_page->footer);
    end_col();
    end_row();
    end_table();
    end_center();
    end_form();
}
