/*
 * Copyright (c) 2005 Kirk Murrin, <mailto:linuxguru1968@yahoo.com>
 * Licensed under the Gnu Public License, version 2.
 * All rights resered.
 * 
 */

#include <string.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <netinet/tcp.h>
#include <syslog.h>
#include <fcntl.h>

#include "request.h"
#include "response.h"
#include "session.h"
#include "html.h"
#include "page_util.h"

#include "make_page.h"

int Pid=-1;

extern char **environ;
extern int errno;
extern html_config_type page_config;
extern sys_config_type sys_config;

#define MAKE_PAGE_ACTION_MAKE          "Make"
#define MAKE_PAGE_ACTION_REFRESH       "Refresh"
#define MAKE_PAGE_ACTION_AUTO_REFRESH  "Auto Refresh"
#define MAKE_PAGE_REFRESH_AUTO_REFRESH "Auto Refresh"
#define MAKE_PAGE_ACTION_STOP_MAKE     "Stop Make"

page_type make_page = {
        "Build Kernel",
        "&nbsp;",
        "&nbsp;",
        handle_make, 
        {
             SELECT,  { "make",      1, "clean dep all zImage bzImage compressed zlilo bzlilo zdisk bzdisk fdimage fdimage144 fdimage288 install modules  modules_prepare modules_install reboot", },
             INPUT,   { "submit",    PARAM_ACTION, MAKE_PAGE_ACTION_MAKE,              },
             INPUT,   { "submit",    PARAM_ACTION, MAKE_PAGE_ACTION_REFRESH,           },
             INPUT,   { "checkbox",  PARAM_REFRESH,MAKE_PAGE_REFRESH_AUTO_REFRESH,  "12", 24, 0, },
             INPUT,   { "submit",    PARAM_ACTION, MAKE_PAGE_ACTION_STOP_MAKE,         },
             NONE,    { "",          "",       "",                  }
        }
};

int handle_make( void )
{
   register int i, found=0;
   int ret=DONT_SEND;
   char cmd[128], *uri_str=NULL, *refresh;
   link_type *tag;
   long rlen;
   char *delims=" \n\t";
   char *method=NULL,*action=NULL;
   char *target=NULL,*menu_level=NULL;
   char *make_val=NULL,*str=NULL,*token=NULL;
   var_value_type *param;
   request_type *request;
   page_table_type *page_table_entry;
   page_type *curr_page;

   page_table_entry=get_request_curr_page();

   request = get_request();
   target = request->target;
   action = request->action;
   method = request->method;

   if ( target  != NULL ) {
       uri_str = strdup(target);
   } else {
       uri_str = strdup("");
   }

   unescape(&uri_str,&rlen);
   set_response_title(uri_str);
   free_string(&uri_str);

   if ( strcasecmp(method,"GET") == 0 ) {
      return setup_make();
   } else {
       if ( strcasecmp(method,"POST") == 0 ) {
           refresh=find_params_value(PARAM_REFRESH);
           if ( refresh ) {
               if ( strcasecmp(refresh,MAKE_PAGE_REFRESH_AUTO_REFRESH) == 0 ) {
                   set_response_refresh(1);
               }
           } else {
               set_response_refresh(0);
           }

           // Update control
           menu_level = page_table_entry->url;
           tag=find_control(menu_level,PARAM_REFRESH);
           if ( tag != NULL ) {
               tag->tag.input.checked=get_response_refresh();
           }
           if ( strcmp(action, MAKE_PAGE_ACTION_MAKE) == 0 ) {
               make_val=find_params_value("make");
               if ( make_val != NULL ) {
                   memset(cmd,0x0,sizeof(cmd));
                   tag=find_control(menu_level,MAKE_PAGE_ACTION_MAKE);
                   str = strdup(tag->tag.select.options);
                   found=0;
                   if ( str ){
                       token=strtok(str,delims);
                       if ( strcmp(token,make_val) == 0 ) {
                           found = 1;
                       }
                       if ( token != NULL ) {
                           while( (token=strtok(NULL,delims)) != NULL ){
                               if ( strcmp(token,make_val) == 0 ) {
                                   found = 1;
                               }
                           }
                       }
                       free(str);
                   }
                   if ( found == 1 ) {
                       if ( strcasecmp(make_val,"reboot") == 0 ) {
                           sprintf(cmd,"(%s)",sys_config.reboot_cmd);
                       } else {
                           sprintf(cmd,"(make %s 2>&1)",make_val);
                       } 
                       Pid = fork_make(cmd);
                   }
               } else {
                   printf("POST action Make but not make value\n"); 
               }
           } else {
               if ( strcmp(action,MAKE_PAGE_ACTION_REFRESH) == 0 ) {
                   // Do nothing just refresh page
               } else {
                   if ( strcmp(action,MAKE_PAGE_ACTION_STOP_MAKE) == 0 ) {
                       if ( Pid != -1 ) {
                           ret=kill(Pid,SIGKILL);
                           Pid=-1;
                       }
                   } else {
                       if ( strcmp(action,MAKE_PAGE_ACTION_AUTO_REFRESH) == 0 ) {
                       }
                   }
               }
           }
           return setup_make();
       }
   }
}

int setup_make( void ) 
{
  int sd;
  struct stat sbuf;
  int len,fd,rc,ret;
  long offset;
  long sendlen, datalen;
  char line[256];
  int saved_nodelay, nodelay;
  int saved_tcp_cork, tcp_cork;
  socklen_t nodelay_size, cork_size;
  FILE *fp;
  char *refresh,*buf;
  response_type *response;

  sd = get_connection_client_sd();

  //set_response_title(Request.uri);
  set_response_title(get_request_target());

  //RTFMP: man pages say this speeds things up???
  nodelay = 0;
  getsockopt(sd,IPPROTO_TCP,TCP_NODELAY,(void*)&saved_nodelay,&nodelay_size);
  setsockopt(sd,IPPROTO_TCP,TCP_NODELAY,(void*)&nodelay,nodelay_size);

  tcp_cork=1;
  getsockopt(sd,IPPROTO_TCP,TCP_CORK,(void*)&saved_tcp_cork,&cork_size);
  setsockopt(sd,IPPROTO_TCP,TCP_CORK,(void*)&tcp_cork,cork_size);

  // Send response header
  set_response_status_code(RC_OK);
  send_status();
  send_date();
  send_cache_control(); 
  send_connection();
  send_server();
  send_content_type();
  send_refresh();
  send_transfer_encoding();
  send_blank_line();

  //send data
  clear_response_data_head();
  start_page();
  start_body();

  response = get_response();

  send_line("%X;\r\n",response->head.datalen);
  rc = send_data(&response->head.data,response->head.datalen);

  clear_response_data_body();
  start_content();
  send_line("%X\r\n",response->body.datalen);
  rc = send_data(&response->body.data,response->body.datalen);

  clear_response_data_body();
  start_iframe(sys_config.build_path,sys_config.build_path, 
               page_config.max_textarea_cols*
               page_config.char_pixel_width,
               page_config.max_textarea_rows*
               page_config.char_pixel_height,0);
  append_page_data_body("If you can see this, your browser doesn't understand IFRAME. \
However, we'll still <a href=\"%c%s\" type=\"text/plain\">link</a> you to the file. \n",path_sep,sys_config.build_path); 
  end_iframe();

  send_line("%X\r\n",response->body.datalen);
  rc = send_data(&response->body.data,response->body.datalen);

  /*
  fd = open(page_config.build_path,O_RDWR);
  if ( fd != -1 ) {
      offset=0L;
      ret=fstat(fd,&sbuf);
      datalen = sbuf.st_size;
      do {
          clear_response_data_body();
          add_comment("START OF TEXTAREA");
          //start_div(page_config.max_textarea_cols*
          //  page_config.char_pixel_width,
          //  page_config.max_textarea_rows*
          //  page_config.char_pixel_height);
          //start_pre();
          start_textarea( page_config.max_textarea_rows,
                          page_config.max_textarea_cols,
                          0,page_config.build_path);
          send_line("%X\r\n",Response.body.datalen);
          rc = send_data(&Response.body.data,Response.body.datalen);

          sendlen = MIN(page_config.max_textarea_file,datalen);
          send_line("%X\r\n",sendlen);
          rc=sendfile_data(fd,&offset,sendlen);
          //rc=sendfile(sd,fd,&offset,sendlen);
          if ( rc < 0 ) {
              // If sendfile fails( for /dev or /proc ??) 
              // copy to userspace and send
              buf=malloc(sendlen);
              if ( buf ) {
                  memset(buf,0x0,sendlen);
                  lseek(fd,offset,SEEK_SET);
                  read(fd,buf,sendlen); 
                  rc=send_data(&buf,sendlen);
                  //rc=send(sd,buf,sendlen,0);
                  free(buf);
              }
          }
          datalen -= sendlen;

          clear_response_data_body();
          end_textarea();
          //end_pre();
          //end_div();
          add_comment("END OF TEXTAREA");
          send_line("%X\r\n",Response.body.datalen);
          rc = send_data(&Response.body.data,Response.body.datalen);
          fsync_data();

      } while ( datalen > 0 ); 

  } else {
      clear_response_data_body();
      start_textarea( page_config.max_textarea_rows,
                      page_config.max_textarea_cols,
                      0,page_config.build_path);
      append_page_data_body("Build outputfile: %s not found",
                       page_config.build_path);
      end_textarea();
      send_line("%X\r\n",response->body.datalen);
      //rc=send(sd, response->body.data,response->body.datalen,0);
      rc=send_data(&response->body.data,response->body.datalen);
      fsync(sd);
  }
  */ 

  clear_response_data_body();
  end_content();
  send_line("%X\r\n",response->body.datalen);
  rc=send_data(&response->body.data,response->body.datalen);
  fsync_data();

  clear_response_data_tail();
  end_body();
  end_page();
  send_line("%X\r\n",response->tail.datalen);
  rc = send_data(&response->tail.data,response->tail.datalen);
  fsync_data();

  // Done with chunked data
  send_line("0\r\n");
  send_blank_line();

  fsync_data();

  setsockopt(sd,IPPROTO_TCP,TCP_NODELAY,(void*)&saved_nodelay,nodelay_size);
  setsockopt(sd,IPPROTO_TCP,TCP_CORK,(void*)&saved_tcp_cork,cork_size);

  return DONT_SEND;

}

int fork_make( char *cmd )
{
   int fd0,fd1,pid;
   if ( cmd ) {
       if ( (pid=fork()) == 0 ) {
           //FIXME:
           fd0=open("/dev/null",O_RDONLY);
           close(0);
           dup(fd0);
           fd1=open(sys_config.build_path,O_WRONLY|O_CREAT|O_TRUNC);
           close(1);
           dup(fd1);
           close(2);
           dup(fd1);
           execlp("/bin/sh","/bin/sh","-c",cmd,(char *)0);
           exit(-1);
       } 
       return pid;
   }
   return -1;
}


void create_build_file( void )
{
    struct stat m;
    int ret;
    FILE *fp;
    memset(&m,0x0,sizeof(m));
    ret=stat(sys_config.build_path,&m);
    if ( ret < 0 ) {
        fp=fopen(sys_config.build_path,"w+");
        if ( fp ) {
            fclose(fp);
        }
    }
}
