/*
 * Copyright (c) 2005 Kirk Murrin, <mailto:linuxguru1968@yahoo.com>
 * Licensed under the Gnu Public License, version 2.
 * All rights resered.
 * 
 */

#include <stdlib.h>
#include <stdio.h>
#include <syslog.h>
#include <libgen.h>

#include "session.h"
#include "response.h"
#include "request.h"
#include "pages.h"
#include "connection.h"

extern page_type main_conf_page;
extern page_type  main_edit_dir_page;
extern page_type main_edit_file_page;
extern page_type main_edit_page;
extern page_type make_page;
extern page_type status_page;
extern page_type logout_page;
extern page_type help_page;

/* URL, Search Order, Menu Order,Visible?, Handler Func */
static page_table_type Pages[] = { 
 MAIN_CONF_URL,     0,0,DISPLAYED,&main_conf_page,
 MAIN_EDIT_FILE_URL,1,6,INVISIBLE,&main_edit_file_page,
 MAIN_EDIT_DIR_URL, 2,7,INVISIBLE,&main_edit_dir_page,
 MAIN_EDIT_URL,     3,1,DISPLAYED,&main_edit_dir_page,
 HELP_URL,          4,5,DISPLAYED,&help_page,
 STATUS_URL,        5,3,DISPLAYED,&status_page,
 LOGOUT_URL,        6,4,DISPLAYED,&logout_page,
 MAKE_URL,          7,2,DISPLAYED,&make_page,
 MAIN_URL,          8,8,INVISIBLE,&main_conf_page,
 ROOT_URL,          9,9,INVISIBLE,&main_conf_page
};

page_table_type  *Default_page = &Pages[0];

int url_in_page_table(char *uri)
{
    register int i;
    if ( strlen(uri) > 0 ) {
        sort_pages_page_order();
        for(i=0; i < sizeof(Pages)/sizeof(Pages[0]); i++ ) {
            if (  memcmp(Pages[i].url,uri,strlen(uri)) == 0 ) {
                return 1;
            }
        }
        syslog(LOG_ERR|LOG_USER,"url_in_page_table: %s not found",uri);
    }
    return 0;
}


page_table_type *get_page_by_url(char *uri)
{
    register int i;
    if ( strlen(uri) > 0 ) {
        // Make sure pages are in order before seaching them 
        sort_pages_page_order();
        for(i=0; i < sizeof(Pages)/sizeof(Pages[0]); i++ ) {
            if (  memcmp(uri,Pages[i].url,strlen(Pages[i].url)) == 0 ) {
                 syslog(LOG_ERR|LOG_USER,"get_page_by_url: Found %s",Pages[i].url);
                 return &Pages[i];
            }
        }
        syslog(LOG_ERR|LOG_USER,"get_page_by_url: %s not found",uri);
    }
    return NULL;
}

int compare_page_page_order(const void *s1, const void *s2)
{
    const page_table_type *p1=(const page_table_type*)s1, *p2=(const page_table_type*)s2;
    return ( p1->page_order - p2->page_order );
}

void sort_pages_page_order( void )
{
    qsort(Pages,(sizeof(Pages)/sizeof(Pages[0])),sizeof(Pages[0]),compare_page_page_order);
}

int compare_pages_menu_order(const void *s1, const void *s2)
{
    const page_table_type *p1=(const page_table_type*)s1, *p2=(const page_table_type*)s2;
    return ( p1->menu_order - p2->menu_order );
}

void sort_pages_menu_order( void )
{
    qsort(Pages,(sizeof(Pages)/sizeof(Pages[0])),sizeof(Pages[0]),compare_pages_menu_order);
}

int compare_page_url(const void *s1, const void *s2)
{
    //int ret,len1,len2;
    const page_table_type *p1=(*(const page_table_type**)s1), *p2=(*(const page_table_type**)s2);
    return !strcasecmp(p1->url,p2->url);
    //return strcasecmp(p1->url,p2->url);
    /*
    len1 = strlen(p1->url);
    len2 = strlen(p2->url);
    ret=strcasecmp(p1->url,p2->url);
    if ( ret == 0 ) {
        return 0;
    }  else {
        if ( ret < 0 ) {
            if ( len2 > len1 ) { 
                return !ret; 
            } else {
                return ret; 
            }
        } else {
            if ( ret > 0 ) {
                if ( len1 > len2 ) {
                    return !ret; 
                } else {
                    return ret; 
                } 
            }
        }
    } 
    */
}

void sort_pages_url( void )
{
    qsort(Pages,(sizeof(Pages)/sizeof(Pages[0])),sizeof(page_type*),compare_page_url);
}

link_type *find_control(char *url, char *ctl_name ) 
{
     int i,j,last_page;
     page_type *page;

     last_page=(sizeof(Pages)/sizeof(Pages[0]));
     for(i=0; i < last_page; i++ ) {
         if ( strcasecmp(Pages[i].url,url) == 0 ) {
            page = Pages[i].page;
            break;
         } 
     }

     if ( i != last_page ) {
         for(j=0; page->tags[j].type != NONE; j++ ) {
             switch( page->tags[j].type) 
             {
                 case ANCHOR:
                     if ( strcasecmp(page->tags[j].tag.link.href,ctl_name) == 0 )
                     {
                          return(&page->tags[j]);
                     }
                 break; 
                 case INPUT:
                     if ( strcasecmp(page->tags[j].tag.input.name,ctl_name) == 0 )
                     {
                          return(&page->tags[j]);
                     }
                 break;
                 case SELECT:
                     if ( strcasecmp(page->tags[j].tag.select.name,ctl_name) == 0 )
                     {
                          return(&page->tags[j]);
                     }
                 break;
                 case LABEL:
                     if ( strcasecmp(page->tags[j].tag.label.text,ctl_name) == 0 )
                     {
                          return(&page->tags[j]);
                     }
                 break;
                 case NONE:
                          return(NULL);
                 break;
                 default:        
                          return(NULL);
                 break;
              }
         }
         
      }
      return NULL;
}

void print_pages( void ) 
{
     int i,j;
     page_type *page;

     for(i=0; i < sizeof(Pages)/sizeof(Pages[0]); i++ ) {
            page = Pages[i].page;
             printf("Address:%p: url: %s page_order: %d menu_order: %d visible: %d\n", 
                     &Pages[i], Pages[i].url,Pages[i].page_order, Pages[i].menu_order, 
                     Pages[i].visible);

             printf("\tAddress: %p %s %s %p\n",
             page, page->header, page->footer, page->handler);

             for(j=0; page->tags[j].type != NONE; j++ ) {
                 switch( page->tags[j].type) 
                 {
                 case ANCHOR:
                 printf("\t%s %s %s\n",
                     page->tags[j].tag.link.type,
                     page->tags[j].tag.link.text, 
                     page->tags[j].tag.link.href); 
                 break; 
                 case INPUT:
                 printf("\t%s %s %s %d %s\n",
                       page->tags[j].tag.input.type,
                       page->tags[j].tag.input.name,
                       page->tags[j].tag.input.size,
                       page->tags[j].tag.input.maxlen,
                       page->tags[j].tag.input.value);
                 break;
                 case SELECT:
                 printf("\t%s %d %s\n",
                      page->tags[j].tag.select.name,
                      page->tags[j].tag.select.size,
                      page->tags[j].tag.select.options);  
                 break;
                 case LABEL:
                 printf("\t%s\n",
                      page->tags[j].tag.label.text);
                 break;
                 case NONE:
                 break;
                 default:        
                 break;
                 }
             }
      }
}

page_table_type *get_default_page( void )
{
    return Default_page;
}

int Page_index;

void init_pages( void )
{
   Page_index = 0;
}

int more_pages( void )
{
    return Page_index < (sizeof(Pages)/sizeof(Pages[0]));
}

void next_page( void )
{
   Page_index++;
}

page_table_type *get_page( void )
{
    return &Pages[Page_index];
}
