/*
 * Copyright (c) 2005 Kirk Murrin, <mailto:linuxguru1968@yahoo.com>
 * Licensed under the Gnu Public License, version 2.
 * All rights resered.
 * 
 */

#ifndef _PAGES_H
#define _PAGES_H

#define PARAM_ACTION  "action"
#define PARAM_TARGET  "target"
#define PARAM_REFRESH "refresh"

// Use this page as default if not found or specified
//#define DEFAULT_PAGE &main_conf_page

#define MAIN_CONF_URL      "/main/conf"
#define MAIN_EDIT_FILE_URL "/main/edit/file"
#define MAIN_EDIT_DIR_URL  "/main/edit/dir"
#define MAIN_EDIT_URL      "/main/edit"
#define HELP_URL	   "/main/help"
#define STATUS_URL         "/main/status"
#define LOGOUT_URL         "/main/logout"
#define MAKE_URL           "/main/make"
#define MAIN_URL           "/main"
#define ROOT_URL           "/"

typedef enum visible_type { INVISIBLE=0, DISPLAYED } visible_type;

typedef enum tag_type {NONE=0, ANCHOR, INPUT, SELECT, LABEL } tag_type;

typedef int (*func_ptr)(void);

//FIXME: use structure packing here ??? or #pragma pack

typedef struct struct_link{
    tag_type type;
    union Tag {
        // Put largest struct first -- DUH!!
        // Looks like compiler or optimizer bug go away with -O > 0 ???
        struct {
            char *type;
            char *name;
            char *value;
            char *size;
            int maxlen;
            int checked;
        } input;
        struct {
            char *name;
            int  size;
            char *options;  
        }select;
        struct {
            char *type;
            char *text;
            char *href;
        } link;
        struct {
            char *text;
        } label;
        // Use padding to make sure all structs covered???
        //char pad[1024];
    } tag;
} link_type;

typedef struct struct_page {
    char *title;
    char *header;
    char *footer;
    func_ptr  handler;
    link_type tags[16];
} page_type;


typedef struct page_table_struct {
    char *url;
    int page_order;
    int menu_order;
    visible_type visible;
    page_type *page;
} page_table_type;

int handle_main_conf( void );
int handle_edit_file(void);
int handle_edit_dir(void);
int handle_help( void );
int handle_status( void );
int handle_make( void );
int handle_logout( void );

int compare_page_menu_order(const void *s1, const void *s2);
void sort_pages_menu_order( void );
int compare_page_url(const void *s1, const void *s2);
void sort_pages_url( void );
int compare_pages_page_order(const void *s1, const void *s2);
void sort_pages_page_order( void );

void print_pages( void );
int url_in_page_table(char *uri);
page_table_type *get_page_by_url(char *uri);
link_type *find_control(char *url, char *ctl_name );

page_table_type *get_default_page( void );

void init_pages( void );
void next_page( void );
int more_pages( void );
page_table_type *get_page( void );

#endif /* */
