/*
 * Copyright (c) 2005 Kirk Murrin, <mailto:linuxguru1968@yahoo.com>
 * Licensed under the Gnu Public License, version 2.
 * All rights resered.
 * 
 */

#include <stdlib.h>
#include <stdarg.h>
#include <syslog.h>
#include <sys/utsname.h>
#include <sys/wait.h>
#include <string.h>
#include <fcntl.h>
#include <libgen.h>
#include <time.h>
#include <unistd.h>

#include "mconfig.h"
#include "response.h"
#include "request.h"
#include "connection.h"
#include "page_util.h"
#include "html.h"

char *Null_ptr = NULL;
extern int errno;

static char WriteLine[MAX_WRITE_LINE];
response_type Response;
//extern connection_type Connection;
extern struct utsname Utsname;
//extern request_type Request;
//extern html_config_type page_config;
extern sys_config_type sys_config;

char *Default_content_type="text/html";
// Page charset encoding: utf-8,utf-7,so-10646-ucs-2,us-ascii,ISO-8859-1 ??
char *Default_charset="ISO-8859-1"; //"utf-8",  

doctype_type Default_doctype = {
"HTML",
"PUBLIC",
"-",
"W3C",
"DTD",
"HTML 4.0 Transitional",
"EN",
"http://www.w3.org/TR/html4/loose.dtd"
};

status_code_type Status_Codes[]=
{
{ RC_SWITCHING_PROTOCOLS,               "Switching Protocols", },
{ RC_PROCESSING,                        "Processing", },
{ RC_OK,                                "OK", },
{ RC_CREATED,                           "Created", },
{ RC_ACCEPTED,                          "Accepted", },
{ RC_NON_AUTHORITATIVE_INFORMATION,     "No Authoritative Information", },
{ RC_NO_CONTENT,                        "Not Content", },
{ RC_RESET_CONTENT ,                    "Reset Content", },
{ RC_PARTIAL_CONTENT,                   "Partial Content", },
{ RC_MULTI_STATUS ,                     "Multi Status", },
{ RC_MULTIPLE_CHOICES ,                 "Multiple Choices", }, 
{ RC_MOVED_PERMANENTLY,                 "Moved Permanently", },
{ RC_SEE_OTHER,                         "See Other", },
{ RC_NOT_MODIFIED,                      "Not Modified", },
{ RC_USE_PROXY,                         "Use Proxy", },
{ RC_TEMPORARY_REDIRECT,                "Temporary Redirect", },
{ RC_BAD_REQUEST,                       "Bad Request", },
{ RC_UNAUTHORIZED,                      "Authorization Required", },
{ RC_PAYMENT_REQUIRED,                  "Payment Required", },
{ RC_FORBIDDEN,                         "Forbidden", },
{ RC_NOT_FOUND,                         "Not Found", },
{ RC_METHOD_NOT_ALLOWED,                "Method Not Allowed", },
{ RC_NOT_ACCEPTABLE,                    "Not Acceptable", },
{ RC_PROXY_AUTHENTICATION_REQUIRED,     "Proxy Authenticateion Required", },
{ RC_REQUEST_TIMEOUT,                   "Request Timeout", },
{ RC_CONFLICT,                          "Conflict", },
{ RC_GONE,                              "Gone", },
{ RC_LENGTH_REQUIRED,                   "Length Required", },
{ RC_PRECONDITION_FAILED,               "Precondition Failed", },
{ RC_REQUEST_ENTITY_TOO_LARGE,          "Request Entity Too Large", },
{ RC_REQUEST_URI_TOO_LARGE,             "Request URI Too Larage", },
{ RC_UNSUPPORTED_MEDIA_TYPE,            "Unsupported Media Type", },
{ RC_REQUEST_RANGE_NOT_SATISFIABLE,     "Request Range Not Satisfiable", },
{ RC_EXPECTATION_FAILED,                "Expectation Failed", },
{ RC_UNPROCESSABLE_ENTITY,              "Unprocessable Entity ", },
{ RC_LOCKED,                            "Locked", },
{ RC_FAILED_DEPENDENCY,                 "Failed Dependency", },
{ RC_INTERNAL_SERVER_ERROR,             "Internal Server Error", },
{ RC_NOT_IMPLEMENTED,                   "Not Implemented", },
{ RC_BAD_GATEWAY,                       "Bad Gateway", },
{ RC_SERVICE_UNAVAILABLE,               "Servie Unavailble" } 
};


void fsync_data( void )
{
   fsync(get_connection_client_sd());
}

int send_data(char **msgptr, size_t len)
{
   int sd,flags=0,rc;
   char *msg=*msgptr; 
   sd = get_connection_client_sd();
   rc = send(sd,msg,len,flags);
   return rc;
}

int sendfile_data(int fd, off_t *offset, size_t count)
{
   int sd,rc;
   sd = get_connection_client_sd();
   rc=sendfile(sd,fd,offset,count);
   return rc;
}

int send_line(char *fmt, ...)
{
   va_list arg_ptr;
   int len,rc;
   int sd;
   sd = get_connection_client_sd();
   va_start(arg_ptr, fmt);
   memset(WriteLine,0x0,sizeof(WriteLine));
   len=vsprintf(WriteLine,fmt,arg_ptr);
   rc=send(sd,WriteLine,len,0);
   va_end(arg_ptr);
   return len;
}

void send_date( void )
{
    int rc;
    time_t curtime;
    struct tm *gm_time;
    int sd;
    sd = get_connection_client_sd();
    curtime = time (NULL);
    gm_time = gmtime(&curtime);
    memset(WriteLine,0x0,sizeof(WriteLine));
    strftime (WriteLine, sizeof(WriteLine), "Date:%a,%d %b %Y %H:%M:%S GMT\r\n", gm_time);
    rc = send(sd,WriteLine,strlen(WriteLine)+1,0);
}


void send_status( void )
{
    status_code_type *status_info=NULL;
    status_info = find_status_code(Response.status_code);
    send_line("HTTP/%2.1f %3d %s\r\n",Response.protocol_ver,
      status_info->status_code,status_info->message); 
}


void send_connection( void )
{
    if ( Response.connection_close ) { 
        send_line("Connection: close\r\n");
    } else {
        //if ( Request.protocol_ver < 1.1 ) {
        //    send_line("Connection: Keep-Alive\r\n");
        //}
    }    
}

void send_cache_control(void )
{
   char str[50];
   memset(str,0x0,sizeof(str));
   if ( Response.cache_control > 0 ) {
       sprintf(str,"max-age=%d",Response.cache_control);
   } else {
       sprintf(str,"no-cache");
   }
   send_line("Cache-Control: %s\r\n",str);
}

void send_server( void )
{
    send_line("Server: %s %s\r\n",argument.cmdname,argument.version);
}

void send_content_type( void )
{
   char str[50];
   memset(str,0x0,sizeof(str));
   sprintf(str,"%s; charset=%s",get_response_content_type(), 
                                 get_response_charset());
   send_line("Content-Type: %s\r\n",str);
}

void send_content_length( long size )
{
    send_line("Content-Length: %ld\r\n",size);
}

void send_expires( void )
{
    send_line("Expires: %d\r\n",Response.expires);
}

void send_pragma( void )
{
   char str[50];
   memset(str,0x0,sizeof(str));
   if ( Response.pragma == 1 ) {
       send_line("Pragma: no-cache\r\n",str);
   } else {
   }
}

void send_refresh( void )
{
  if ( Response.refresh ) {
          send_line("Refresh: %ld\r\n",sys_config.page_refresh_time);
  } else {
  }

}

void send_transfer_encoding( void )
{
  send_line("Transfer-Encoding: chunked\r\n");
}

void send_blank_line( void )
{
  send_line("\r\n");
}

void send_response( void )
{
  int fd,rc;
  cookie_type *cookie;
  page_data_type *head,*body,*tail;

  send_status(); 
  send_server();
  send_date();
  send_connection();
  send_cache_control();
  //send_expires();
  //send_pragma();
  send_content_type();
  send_transfer_encoding();
  send_blank_line();
 
  send_line("%X;\r\n",Response.head.datalen);
  rc = send_data(&Response.head.data,Response.head.datalen);

  send_line("%X\r\n",Response.body.datalen);
  rc = send_data(&Response.body.data,Response.body.datalen);

  send_line("%X\r\n",Response.tail.datalen);
  rc = send_data(&Response.tail.data,Response.tail.datalen);

  send_line("0\r\n");

  send_blank_line();

}

int send_challenge( void )
{
  int rc;
  set_response_status_code(RC_UNAUTHORIZED);
  send_status();
  send_server();
  send_date();
  send_cache_control();
  send_content_type();

  // Send request for authentication; put scheme in page_config??? 
  send_line("WWW-Authenticate: Basic realm=\"%s\"\r\n",argument.title);
  //send_line("WWW-Authenticate: Digest realm=\"biloxi.com\", qop=\"auth,auth-int\", nonce=\"dcd98b7102dd2f0e8b11d0f600bfb0c093\", opaque=\"5ccc069c403ebaf9f0171e9517f40e41\"");

  //if ( Session && Session->session_id ) {
  //   send_line("set-cookie: sessionid=%s\r\n",Session->session_id);
  //}

  send_transfer_encoding();
  send_blank_line();

  clear_page_data(&Response.head);
  set_response_title("Authorization Required"); 
  start_page();
  start_body();

  clear_page_data(&Response.body);
  append_page_data(&Response.body,"<h1>Authorization Required</h1> \"%s\" could not verify that you are authorized to access the document you requested. \
Either you suppplied the wrong credentials( e.g., bad password) or your browser dosen't understand how to supply the \
credentials requried<p><hr><b>%s %s %s %s %s %s</b><br></body</html>",
      argument.title,argument.cmdname,argument.version,
      Utsname.sysname,Utsname.nodename,Utsname.release,Utsname.machine);

  clear_page_data(&Response.tail);
  end_body();
  end_page();


  send_line("%X;\r\n",Response.head.datalen);
  rc = send_data(&Response.head.data,Response.head.datalen);

  send_line("%X\r\n",Response.body.datalen);
  rc = send_data(&Response.body.data,Response.body.datalen);

  send_line("%X\r\n",Response.tail.datalen);
  rc = send_data(&Response.tail.data,Response.tail.datalen);

  send_line("0\r\n");
  send_blank_line();

  return 1;

}

int append_file_data(FILE *fp, char *fmt, ...)
{
   int ret;
   va_list arg_ptr;
   int len;
   va_start(arg_ptr, fmt);
   memset(WriteLine,0x0,sizeof(WriteLine));
   len=vsprintf(WriteLine,fmt,arg_ptr);
   ret=fwrite(WriteLine,len,1,fp);
   va_end(arg_ptr);
   return len;
}


int append_page_data(page_data_type *data, char *fmt, ...)
{
   va_list arg_ptr;
   int len;
   va_start(arg_ptr, fmt);
   memset(WriteLine,0x0,sizeof(WriteLine));
   len=vsprintf(WriteLine,fmt,arg_ptr);
   data->data = realloc(data->data,data->datalen+len);
   memset(&data->data[data->datalen],0x0,len);
   memcpy(&data->data[data->datalen],WriteLine,len);
   data->datalen += len;
   va_end(arg_ptr);
   return len;
}


int append_page_data_head(char *fmt, ...)
{
   va_list arg_ptr;
   int len;
   va_start(arg_ptr, fmt);
   memset(WriteLine,0x0,sizeof(WriteLine));
   len=vsprintf(WriteLine,fmt,arg_ptr);
   Response.head.data = realloc(Response.head.data, Response.head.datalen+len);
   memset(&Response.head.data[Response.head.datalen],0x0,len);
   memcpy(&Response.head.data[Response.head.datalen],WriteLine,len);
   Response.head.datalen += len;
   va_end(arg_ptr);
   return len;
}


int append_page_data_body(char *fmt, ...)
{
   va_list arg_ptr;
   int len;
   va_start(arg_ptr, fmt);
   memset(WriteLine,0x0,sizeof(WriteLine));
   len=vsprintf(WriteLine,fmt,arg_ptr);
   Response.body.data = realloc(Response.body.data,Response.body.datalen+len);
   memset(&Response.body.data[Response.body.datalen],0x0,len);
   memcpy(&Response.body.data[Response.body.datalen],WriteLine,len);
   Response.body.datalen += len;
   va_end(arg_ptr);
   return len;
}


int append_page_data_tail(char *fmt, ...)
{
   va_list arg_ptr;
   int len;
   va_start(arg_ptr, fmt);
   memset(WriteLine,0x0,sizeof(WriteLine));
   len=vsprintf(WriteLine,fmt,arg_ptr);
   Response.tail.data = realloc(Response.tail.data,Response.tail.datalen+len);
   memset(&Response.tail.data[Response.tail.datalen],0x0,len);
   memcpy(&Response.tail.data[Response.tail.datalen],WriteLine,len);
   Response.tail.datalen += len;
   va_end(arg_ptr);
   return len;
}

void clear_page_data(page_data_type *data)
{
    if ( data->data != NULL ) {
        free(data->data);
    }
    data->data=NULL;
    data->datalen = 0; 
}

void clear_response_data_head(void)
{
   clear_page_data(&Response.head);
}

void clear_response_data_body(void)
{
   clear_page_data(&Response.body);
}

void clear_response_data_tail(void)
{
   clear_page_data(&Response.tail);
}

void clear_response( void )
{
   //status_code_type *status_info;
   request_type *request;

   request = get_request();

   clear_page_data(&Response.head);
   clear_page_data(&Response.body);
   clear_page_data(&Response.tail);

   //FIXME: Right now use request proto ver - later use logic
   Response.protocol_ver=request->protocol_ver;
   Response.status_code = RC_OK;

   memcpy(&Response.doctype,&Default_doctype,sizeof(Response.doctype));

   free_string(&Response.page_title);
   reset_string_value(&Response.charset,&Default_charset);
   reset_string_value(&Response.content_type,&Default_content_type);

   Response.expires=0;
   Response.cache_control=0;
   Response.pragma=0;
   Response.refresh=0;
   Response.append=0;
   Response.error_found=0;

   if ( get_protocol_ver() < 1.1 ) {
       Response.connection_close=1;
   } else {
       Response.connection_close=0;
   }

}

void print_response_data (void)
{
    //FIXME: chunk to prevent stack overflow.
    fwrite(Response.head.data,Response.head.datalen ,1,stdout);
    fwrite(Response.body.data,Response.body.datalen ,1,stdout);
    fwrite(Response.tail.data,Response.tail.datalen ,1,stdout);
}

void print_response_doctype( void )
{
    printf("DOCTYPE:\n top_element: %s availability: %s\n  registration: %s \n organization: %s \n type: %s \n label: %s \n language:%s \n fpi: %s\n",
                  Response.doctype.top_element,
                  Response.doctype.availability,
                  Response.doctype.registration,
                  Response.doctype.organization,
                  Response.doctype.type,
                  Response.doctype.label,
                  Response.doctype.language,
                  Response.doctype.fpi);
}

void print_response( void )
{
   //int i;
   printf("\n\nRESPONSE:\n");
   printf("---------------------------------------------------------\n");
   printf("PROTOCOL: %f\n",Response.protocol_ver);
   printf("STATUS CODE: %3d\n",Response.status_code);
   printf("CONTENT_TYPE: %s\n",Response.content_type);
   printf("REFRESH: %d\n",Response.refresh);
   printf("APPEND: %d\n",Response.append);
   printf("CONNECTION_CLOSE: %d\n",Response.connection_close);
   print_response_data();
   print_response_doctype();
   printf("\n---------------------------------------------------------\n");
}

status_code_type *find_status_code( int status_code )
{
    register int i;
    for(i=0; i < (sizeof(Status_Codes)/sizeof(Status_Codes[0])); i++ ) {
        if ( Status_Codes[i].status_code == status_code ) {
            return &Status_Codes[i];
        }
    }
    return NULL;
}


//FIXME: Use libfile to create proper mapping of file to mime type
char *file_to_mime( char *filename )
{
      char *basec=NULL,*dirc=NULL,*bname=NULL,*dname=NULL;
      char name[NAME_MAX],ext[NAME_MAX];
      char *mime_type=NULL;

      dirc  = strdup(filename);
      basec = strdup(filename);
      dname = dirname(dirc);
      bname = basename(basec);

      memset(name,0x0,sizeof(name));
      memset(ext,0x0,sizeof(ext));

      sscanf(bname,"%[^.].%s",name,ext);

      free(dirc);
      free(basec);

      if ( strlen(ext) != 0 ) {
          if ( strcasecmp(ext,"jpg") == 0 || strcasecmp(ext,"jpeg") == 0) {
              mime_type = strdup("image/jpeg");
          } else {
              if ( strcasecmp(ext,"gif") == 0 ) {
                  mime_type = strdup("image/gif");
              } else {
                  if ( strcasecmp(ext,"htm") == 0 || strcasecmp(ext,"html") == 0 ) {
                      mime_type = strdup("text/html");
                  } else {
                      if ( strcasecmp(ext,"txt") == 0 ) {
                          mime_type = strdup("text/plain");
                      } else {
                          mime_type = strdup("text/plain");
                      }
                  }
              }
          }
      } else {
          mime_type = strdup("text/plain");
      }
      return mime_type;
}

int setup_error(int status_code,char *fmt, ...)
{
    char message[128 ];
    va_list arg_ptr;
    int len;
    status_code_type *status_info=NULL;


    va_start(arg_ptr, fmt);
    memset(message,0x0,sizeof(message));
    len=vsprintf(message,fmt,arg_ptr);
    va_end(arg_ptr);

    clear_response();

    set_response_status_code(status_code);

    status_info = find_status_code(status_code);

    if ( status_info == NULL ) {
       // Set a default here
    } 
    set_response_title("%d %s",status_code,status_info -> message);
    start_page();
    start_body();

    append_page_data_body("<h1>%s</h1>",status_info -> message);

    append_page_data_body("<h2>%s</h2>",message);

    append_page_data_body("<hr><b>%s&nbsp;%s&nbsp;%s&nbsp;%s&nbsp;%s&nbsp;%s&nbsp; %s</b>\n<br>\n",
            argument.title,
            argument.cmdname,
            argument.version,
            Utsname.sysname,
            Utsname.nodename,
            Utsname.release,
            Utsname.machine);
    

    end_body();
    end_page();
    return SEND_PAGE;
}


int send_file( char *filename ) 
{
  FILE  *fp;
  int ret,pid=!0;
  struct stat buf;
  long  offset=0, cnt;
  long sendlen;
  char *content_type;
  int sd,child_exit_status,ws;

  syslog(LOG_ERR|LOG_USER,"send_file: start %s",filename);

  //fcntl(sd, F_SETFD,~FD_CLOEXEC&fcntl(sd,F_GETFD));

  memset(&buf,0x0,sizeof(buf));

  ret=stat(filename,&buf);

  if ( ret < 0  ){
      setup_error(RC_NOT_FOUND,"Requested file %s not found: %s",
                  filename,strerror(errno));
      return -1;
  }

  if ( (pid=fork()) == 0 ) {
      sd = dup(get_connection_client_sd());
      content_type = file_to_mime(filename);
      set_response_content_type(content_type);
      free(content_type);
      fp = fopen(filename,"r+");
      if ( fp ) {
          set_response_status_code(RC_OK);
          send_status();
          send_server();
          send_date();
          send_cache_control();
          send_connection();
          send_content_type();
          send_content_length(buf.st_size);
          send_blank_line();
          sendlen = buf.st_size;
          offset = 0;
          do {
              cnt=sendfile(sd,fileno(fp),&offset,sendlen);
              sendlen -= cnt;
          } while( sendlen > 0 );
          fclose(fp);
      }
      _exit(0);
  }
  child_exit_status=0;
  ws = waitpid(pid,&child_exit_status,0);
  if( WIFEXITED(child_exit_status) ) {
      syslog(LOG_ERR|LOG_USER,"send_file: chiled extied normally");
  } else {
      syslog(LOG_ERR|LOG_USER,"send_file: chiled extied abnormally");
  }
  return pid;
}

void start_inline_send( void )
{
    int rc;
    set_response_status_code(RC_OK);
    send_status();
    send_date();
    send_cache_control();
    send_server();
    send_content_type();
    send_transfer_encoding();
    send_blank_line();

    clear_page_data(&Response.head);
    start_page();
    start_body();
    send_line("%X;\r\n",Response.head.datalen);
    rc = send_data(&Response.head.data,Response.head.datalen);

    clear_page_data(&Response.body);
    start_content();
    send_line("%X\r\n",Response.body.datalen);
    rc=send_data(&Response.body.data,Response.body.datalen);

    fsync_data();

}


int end_inline_send( void )
{
    int rc;

    clear_page_data(&Response.body);
    end_content();
    send_line("%X\r\n",Response.body.datalen);
    rc=send_data(&Response.body.data,Response.body.datalen);

    clear_page_data(&Response.tail);
    end_body();
    end_page();
    send_line("%X\r\n",Response.tail.datalen);
    rc=send_data(&Response.tail.data,Response.tail.datalen);

    send_line("0\r\n");
    send_blank_line();

    fsync_data();

    return DONT_SEND;
}

void set_response_title(char *fmt, ...)
{
   va_list arg_ptr;
   char tmpstr[128];
   char title[128];
   char hostname[128];
   int len;
   request_type *request=NULL;

   request = get_request();

   memset(tmpstr,0x0,sizeof(tmpstr));
   memset(title,0x0,sizeof(title));
   memset(hostname,0x0,sizeof(hostname));

   gethostname(hostname,sizeof(hostname));
   va_start(arg_ptr, fmt);
   len=vsprintf(tmpstr,fmt,arg_ptr);
   free_string(&Response.page_title);
   sprintf(title,"%s: %s %s",hostname,
                (request->curr_page!=NULL?request->curr_page->page->title:""),tmpstr);
   Response.page_title = strdup(title);
   va_end(arg_ptr);
}

response_type *get_response( void )
{
   return &Response;
}

char *get_response_title( void )
{
   return Response.page_title;
}

void set_response_cache_control( int seconds )
{
    Response.cache_control=seconds;
}

int  get_response_cache_control( void )
{
    return Response.cache_control;
}

void set_response_expires( int seconds )
{
    Response.expires=seconds;
}

int get_response_expires( void )
{
    return Response.expires;
}

int get_response_pragma( void )
{
    return Response.pragma;
}

char *get_response_content_type( void )
{
    return Response.content_type;
}

void set_response_content_type( char *content_type )
{
    reset_string_value(&Response.content_type,&content_type);
}

int get_response_error_found( void )
{
    return Response.error_found;
}

void set_response_error_found( int error_found )
{
    Response.error_found = error_found;
}

int get_response_connection_close( void )
{
    return Response.connection_close;
}

void set_response_connection_close( int close )
{
    Response.connection_close=close;
}

float get_response_protocol_ver( void )
{
    return Response.protocol_ver;
}

doctype_type *get_response_doctype( void )
{
    return &Response.doctype;
}

char *get_response_charset( void )
{
    return Response.charset;
}

page_data_type *get_response_head( void )
{
    return &Response.head;
}

page_data_type *get_response_body( void )
{
    return &Response.body;
}

page_data_type *get_response_tail( void )
{
    return &Response.tail;
}

void set_response_refresh( int refresh )
{
    Response.refresh= refresh;
}

int get_response_refresh( void ) 
{
    return Response.refresh;
}

int get_response_status_code( void )
{
    return Response.status_code;
}

void set_response_status_code( int status_code )
{
    Response.status_code = status_code;
}

int get_response_append( void )
{
    return Response.append;
}

void set_response_append( int append )
{
    Response.append=append;
}
