/*
 * Copyright (c) 2005 Kirk Murrin, <mailto:linuxguru1968@yahoo.com>
 * Licensed under the Gnu Public License, version 2.
 * All rights resered.
 * 
 */

#include "mconfig.h"

#include <stdio.h>
#include <sys/sendfile.h>


#ifndef _RESPONSE_H
#define _RESPONSE_H

#define MAX_WRITE_LINE   512  // Max size of output line - HTML code

#define   RC_CONTINUE                          100
#define   RC_SWITCHING_PROTOCOLS               101
#define   RC_PROCESSING                        102
#define   RC_OK                                200
#define   RC_CREATED                           201
#define   RC_ACCEPTED                          202
#define   RC_NON_AUTHORITATIVE_INFORMATION     203
#define   RC_NO_CONTENT                        204
#define   RC_RESET_CONTENT                     205
#define   RC_PARTIAL_CONTENT                   206
#define   RC_MULTI_STATUS                      207
#define   RC_MULTIPLE_CHOICES                  300
#define   RC_MOVED_PERMANENTLY                 301
#define   RC_FOUND                             302
#define   RC_SEE_OTHER                         303
#define   RC_NOT_MODIFIED                      304
#define   RC_USE_PROXY                         305
#define   RC_TEMPORARY_REDIRECT                307
#define   RC_BAD_REQUEST                       400
#define   RC_UNAUTHORIZED                      401
#define   RC_PAYMENT_REQUIRED                  402
#define   RC_FORBIDDEN                         403
#define   RC_NOT_FOUND                         404
#define   RC_METHOD_NOT_ALLOWED                405
#define   RC_NOT_ACCEPTABLE                    406
#define   RC_PROXY_AUTHENTICATION_REQUIRED     407
#define   RC_REQUEST_TIMEOUT                   408
#define   RC_CONFLICT                          409
#define   RC_GONE                              410
#define   RC_LENGTH_REQUIRED                   411
#define   RC_PRECONDITION_FAILED               412
#define   RC_REQUEST_ENTITY_TOO_LARGE          413
#define   RC_REQUEST_URI_TOO_LARGE             414
#define   RC_UNSUPPORTED_MEDIA_TYPE            415
#define   RC_REQUEST_RANGE_NOT_SATISFIABLE     416
#define   RC_EXPECTATION_FAILED                417
#define   RC_UNPROCESSABLE_ENTITY              422
#define   RC_LOCKED                            423
#define   RC_FAILED_DEPENDENCY                 424
#define   RC_INTERNAL_SERVER_ERROR             500
#define   RC_NOT_IMPLEMENTED                   501
#define   RC_BAD_GATEWAY                       502
#define   RC_SERVICE_UNAVAILABLE               503
#define   RC_GATEWAY_TIMEOUT                   504
#define   RC_HTTP_VERSION_NOT_SUPPORTED        505
#define   RC_INSUFFICIENT_STORAGE              507

typedef enum enum_page_status {SEND_PAGE=0,DONT_SEND} page_status_type;

typedef struct status_struct {
	int status_code;
        char *message;
} status_code_type;

typedef struct doctype
{
char *top_element;
char *availability;
char *registration;
char *organization;
char *type;
char *label;
char *language;
char *fpi;
} doctype_type;

typedef struct struct_page_data {
    char *data;
    long datalen;
} page_data_type;

typedef struct struct_response {
    float 	     protocol_ver;
    doctype_type     doctype;
    char             *charset;
    int   	     status_code;
    char  	     *content_type;
    int              expires;
    int              cache_control;
    int              pragma;
    int              connection_close;
    char             *page_title;
    page_data_type   head;
    page_data_type   body;
    page_data_type   tail;
    int refresh;
    int append;
    int error_found;
    //cookie_type *cookie;
} response_type;


void fsync_data( void );
void send_response( void );
void start_inline_send( void );
int end_inline_send( void );
int send_data(char **msgptr, size_t len);
int sendfile_data(int fd, off_t *offset, size_t count);
int  send_line(char *fmt, ...);
void send_date( void );
void send_server( void );
void send_content_type( void );
void send_status( void );
void send_expires( void );
void send_pragma( void );
void send_connection( void );
void send_refresh( void );
void send_transfer_encoding( void );
void send_blank_line( void );

int check_connection( void );
int send_file( char *filename );
int send_challenge( void );
void clear_response(void);
void print_response( void );
void print_response_doctype( void );
void print_response_data(void);
void clear_page_data(page_data_type *data);
int append_page_data(page_data_type *data, char *fmt, ...);
int append_page_data_head(char *fmt, ...);
int append_page_data_body(char *fmt, ...);
int append_page_data_tail(char *fmt, ...);
void clear_response_data_head(void);
void clear_response_data_body(void);
void clear_response_data_tail(void);
int  append_file_data(FILE *fp, char *fmt, ...);
void start_inline_send( void );
int end_inline_send( void );

status_code_type *find_status_code( int status_code );
char *file_to_mime( char *filename );
//int authenticate_session( void );

void set_response_title(char *fmt, ...);
char *get_response_title( void );
int  get_response_expires( void );
int get_response_pragma( void );
char *get_response_content_type( void );
void set_response_content_type( char *content_type );
int get_response_error_found( void );
void set_response_error_found( int error_found );
response_type *get_response( void );
int get_response_connection_close( void );
void set_response_connection_close( int close );
float get_response_protocol_ver( void );
doctype_type *get_response_doctype( void );
char *get_response_charset( void );
page_data_type *get_response_head( void );
page_data_type *get_response_body( void );
page_data_type *get_response_tail( void );
void set_response_cache_control( int seconds );
int get_response_cache_control( void );
void set_response_expires( int seconds );
int  get_response_expires( void );
void set_response_refresh( int refresh );
int get_response_refresh( void );
void send_content_length( long size );
int get_response_status_code( void );
void set_response_status_code( int status_code );
int get_response_append( void );
void set_response_append( int append );
int setup_error(int status_code,char *fmt, ...);

#endif /* */
