/*
 * Copyright (c) 2005 Kirk Murrin, <mailto:linuxguru1968@yahoo.com>
 * Licensed under the Gnu Public License, version 2.
 * All rights resered.
 * 
 */

#include <string.h>
#include <pwd.h>
#include <syslog.h>

#if !defined(USE_PAM)
#include <shadow.h>
#include <pwauth.h>
#endif

#include "mconfig.h"

#include "session.h"
#include "base64.h"
#include "request.h"
#include "response.h"
#include "connection.h"

extern sys_config_type sys_config;

auth_table_type Auth_Schemes[]= {
    { "Basic", auth_basic }
};

session_type *Session;

void init_session(void)
{
    if ( Session ) clear_session();
    Session = malloc(sizeof(session_type));
    memset(Session,0x0,sizeof(session_type));
    Session->authorization=NULL;
    Session->auth_info.scheme=NULL;
    Session->auth_info.auth_data=NULL;
    Session->auth_info.usr_pass=NULL;
    Session->auth_info.username=NULL;
    Session->auth_info.password=NULL;
    Session->addr_len=0;
    memset(&Session->client_addr,-1,Session->addr_len);
    Session->session_id=NULL;
    Session->random_num = 0;
    Session->seq_num = 0;
    Session->authenticated=0;
    Session->challenge_sent=0;
    Session -> login_cnt=0;
    Session->last_login_time=-1;
    Session->mstate=NULL;
    if ( parser_block_top ) {
        Session->block=parser_block_top;
        if ( parser_block_top->first ) {
            Session->statement=parser_block_top->first;
        }
    }
    Session->cwd=strdup("/");
}

void clear_session( void )
{

  if ( Session ) { 


      free_string(&Session->authorization);
      free_string(&Session->auth_info.scheme);
      free_string(&Session->auth_info.auth_data);
      free_string(&Session->auth_info.usr_pass);
      free_string(&Session->auth_info.username);
      free_string(&Session->auth_info.password);
      free_string(&Session->session_id);
      Session->random_num = 0;
      Session->seq_num = 0;

      Session->login_cnt=0;
      Session->last_login_time=-1;

      //Session->addr_len=0;
      //memset(&Session->client_addr,-1,Session->addr_len);
      //Session->authenticated=0;
      //Session->challenge_sent=0;

      //Session->mstate=NULL;
      //Session->block=NULL;
      //Session->statement=NULL;
      //if ( Session->cwd ) {
       //  free(Session->cwd);
        // Session->cwd=NULL;
      //}

  } else {
     init_session();
  }

}

char *set_session_id( void )
{
    char *session_id=NULL;
    long rnum=0;
    char snum[16];

    if ( Session ) {
        Session -> seq_num++;
        srand(Session->seq_num);
        Session->random_num=rand();
        memset(snum,0x0,sizeof(snum));
        sprintf(snum,"%ld",Session->random_num);
        session_id=base64_encode_string(snum);
        reset_string_value(&Session -> session_id,&session_id);
        return Session -> session_id;
    }
    return NULL;
}

int check_sessionid( void )
{
    char *clear_sessionid=NULL;
    char *sessionid;
    long rndnum;
    if ( Session ) {
        sessionid=find_cookie_value("sessionid");
        if ( sessionid ) {
            if ( Session->random_num > 0 ) {
                clear_sessionid=base64_decode_string(sessionid);
                rndnum=atol(clear_sessionid);
                if ( rndnum != Session->random_num ) {
                   return -1;
                } else {
                   return 0;
                }
            }
        }
    }
    return 0;
}

char *session_get_cwd( void )
{
    return Session -> cwd;
}

void session_set_cwd( char *pathname ) 
{
    char cwdpath[PATH_MAX],newcwdpath[PATH_MAX];
    char *pathptr=NULL;
    int  ret;
    if ( strlen(pathname)  > 0 && Session ) {
        memset(cwdpath,0x0,sizeof(cwdpath));
        getcwd(cwdpath,sizeof(cwdpath));
        ret=chdir(pathname);
        if ( ret == 0 ) {
            memset(newcwdpath,0x0,sizeof(newcwdpath));
            getcwd(newcwdpath,sizeof(newcwdpath));
            pathptr=strdup(newcwdpath);
            reset_string_value(&Session->cwd,&pathptr);
            ret=chdir(cwdpath);
            free(pathptr);
        }
    }
}

void set_statement_default( void )
{
   if ( Session != NULL ) {
       Session->statement=Session->block->first;
   }
}

session_type *get_session( void )
{
    return Session;
}


mstate_type *get_session_mstate( void )
{
    return Session->mstate;
}


void init_stack( void )
{
    /* init menu state */
    if ( Session ) {
        Session->mstate = check_malloc(sizeof(mstate_type));
        memset(Session->mstate,0x0,sizeof(*Session->mstate));
        Session->mstate->parent = NULL;
        Session->mstate->menu = Session->block->first;
        Session->mstate->ishelp = 0;
        Session->mstate->plist.list=NULL;
        Session->mstate->plist.len  = 0;
    }
}

int is_top_menu( void )
{
    return (Session->mstate->menu == Session->block->first);
}

void push_statement( void )
{
    mstate_type  *mstate=NULL;
    if ( Session ) {
        mstate = Session->mstate;
        printf("Pushing %p onto menu stack\n",Session->statement);
        mstate = check_malloc(sizeof(*mstate));
        mstate->parent = Session->mstate;
        Session->mstate = mstate;
        mstate->menu = Session->statement;
        mstate->ishelp = 0;
        mstate->plist.list = NULL;
        mstate->plist.len  = 0;
    }
}

void pop_statement( void )
{
    mstate_type  *mstate=NULL;
    if ( Session ) {
        mstate = Session->mstate;
        printf("Popping %p off menu stack\n",mstate->menu);
        if ( mstate->parent != NULL ) {
            Session->mstate = mstate->parent;
            if ( mstate->plist.list ) free(mstate->plist.list);
            mstate->plist.list=NULL;
            mstate->plist.len  = 0;
            free(mstate);
        }
    }
}

void pop_to_top( void )
{
    mstate_type  *mstate;
    if ( Session ) {
        mstate=Session->mstate;
        if ( mstate ) {
            while( (mstate)->parent != NULL ){
                Session->mstate = mstate->parent;
	        if ( mstate->plist.list ) free(mstate->plist.list);
	        mstate->plist.list=NULL;
	        mstate->plist.len  = 0;
	        free(mstate);
                mstate = Session->mstate; 
            }
            Session->statement=NULL;
        }
    }
}

void free_stack_entry( mstate_type  *mstate)
{ 
    if ( mstate ) {
        if ( mstate->plist.list ) free(mstate->plist.list);
        mstate->plist.list=NULL;
        mstate->plist.len  = 0;
        free(mstate);
    }
}

int session_challenge_sent( void )
{
    return (Session->challenge_sent == 1 );
}

void session_set_challenge_sent( int sent )
{
    Session->challenge_sent = sent;
}

int session_authenticated( void )
{
    return (Session->authenticated==1);
}

void session_set_authenticated( int authenticated )
{
    Session->authenticated = authenticated;
}


int process_authenticate( void )
{
    int ret=0;
    if ( !session_challenge_sent() && 
         !session_authenticated()  ){
        if ( session_check_login() ) {
            send_challenge();
            session_set_challenge_sent(1);
            session_update_login_cnt();
            session_set_last_login_time();
            return CHALLENGE_SENT;
        } else {
             syslog(LOG_ERR|LOG_USER,
                    "process_authenticate: Passwd cracker!! Exiting. Call Kevin Mitnik");
             printf("Someone using my programs as a passwd cracker!!\n");
             exit(-1);
        }
    } else {
        ret=verify_credentials();
        if ( ret == 0 ) {
            session_set_authenticated(1);
            session_set_challenge_sent(0);
            session_reset_login_cnt();
            session_reset_last_login_time();
            return AUTH;
        } else {
            session_set_challenge_sent(0);
            session_set_challenge_sent(0);
            session_update_login_cnt();
            session_set_last_login_time();
            return NOTAUTH;
        }
    }
    return NOTAUTH;
}

int auth_basic( void )
{
    char *ptr1,*colonptr;
    struct passwd *pw;
    char *epasswd;

#if !defined(USE_PAM)
    struct spwd *spwd;
    struct spwd *getspnam();
#endif

    if ( Session->auth_info.auth_data == NULL ) {
            return NOTAUTH; 
    }

    Session->auth_info.usr_pass=base64_decode_string(Session->auth_info.auth_data);

    if ( Session->auth_info.usr_pass == NULL ) {
            return NOTAUTH;
    }

    ptr1=Session->auth_info.usr_pass;
    colonptr=strchr(ptr1,':');

    if ( colonptr == NULL ) {
        return NOTAUTH;
    }

    Session->auth_info.username=(char*)strndup(ptr1,colonptr-ptr1);
    Session->auth_info.password=(char*)strdup(colonptr+1);

    // If not root user, let them build and install kernel???
    if ( strcasecmp(Session->auth_info.username,"root") != 0 ) {
        return NOTAUTH;
    }

    if ((pw = getpwnam(Session->auth_info.username)) == NULL) {
        return NOTAUTH;
    }

#if !defined(USE_PAM)
    spwd = getspnam(Session->auth_info.username);
    if (spwd){
        //memory leak??
        pw->pw_passwd = spwd->sp_pwdp;
    }
#endif

    // If no passwd dont let them login in
    if (pw->pw_passwd == '\0') {
        return NOTAUTH;
    }

#if defined(USE_PAM)
    epasswd = crypt(Session->auth_info.password, pw->pw_passwd);
    if (strcmp(epasswd, pw->pw_passwd)) {
        return NOTAUTH;
    }
#else
    if ( pw_auth(pw->pw_passwd,pw->pw_name,PW_LOGIN,Session->auth_info.password) != 0 )
    {
        return NOTAUTH;
    }
#endif

    return AUTH;

}

// Return 0 if authenticated; -1 otherwise
int verify_credentials( void ) 
{
    char *authorization=NULL;
    char *ptr1,*ptr2,*ptr3;
    long rndnum=0;
    int ret=0;
    connection_type *connection;
    register int i;
    int (*auth_handler)(void);

    connection = get_connection();

    authorization=find_header_value("Authorization");
    if ( authorization != NULL ) {
        clear_session();
        Session->authorization=authorization;
        Session->addr_len=connection->addr_len;
        memcpy(&Session->client_addr,&connection->client_addr,Session->addr_len);
        ptr1=Session->authorization;
        ptr2=strchr(ptr1,' ');
        if ( ptr2 != NULL ) {
            ptr3 = (char*)(ptr2-ptr1);
            Session->auth_info.scheme=(char*)strndup(ptr1,ptr3);
            while ( isspace(*ptr2) ) ptr2++;
            Session->auth_info.auth_data=strdup(ptr2);
        }

        ret=NOTAUTH;
        auth_handler=NULL;
        for(i=0; i < sizeof(Auth_Schemes)/sizeof(Auth_Schemes[0]);i++) {
            if ( strcasecmp(Auth_Schemes[i].scheme,Session->auth_info.scheme) == 0 ) {
                auth_handler=Auth_Schemes[i].auth_handler;
                break;
            }
        }
        if ( auth_handler ) {
            ret=auth_handler();
        } else {
            setup_error(RC_NOT_IMPLEMENTED,"Authorizatin Scheme %s not supported",
                        Session->auth_info.scheme);
            clear_session();
            return -1;
        }

        if ( ret == NOTAUTH ) { 
            setup_error(RC_INTERNAL_SERVER_ERROR,"Authorization cannot be validated");
            clear_session();
            return -1;
        } else {
            ret=check_sessionid();
            if ( ret < 0 ) {
                setup_error(RC_INTERNAL_SERVER_ERROR,"Invalid Session ID");
                clear_session();
                return -1;
            }
            set_session_id();
            return 0;
        }
    } else {
        setup_error(RC_INTERNAL_SERVER_ERROR,"Response did not contain authorization");
        clear_session();
        return -1;
    }
    return -1;
}

block_type *session_get_block_top( void )
{
    return Session->block;
}

statement_type *session_get_statement( void )
{
    return Session->statement;
}

int session_set_statement( char *pointer_str )
{
    return(sscanf(pointer_str,"%p",&Session->statement) == 1);
}

void session_set_statement_block_first( void )
{
    Session->statement=Session->block->first;
}

/*
void session_statement_sb_symbol_value(piece_type piece )
{
    Session->statement->sb_symbol->value = piece;
}

void session_statement_sb_symbol_origin(origin_type origin_statement )
{
    Session->statement->sb_symbol->origin = origin_statement;
}
*/

long get_session_s_addr( void )
{
    return Session->client_addr.sin_addr.s_addr;
}

void session_update_login_cnt( void )
{
    Session -> login_cnt++;
}

int session_get_login_cnt( void )
{
    return Session -> login_cnt;
}

void session_reset_login_cnt( void )
{
    Session -> login_cnt=0;
}

void session_set_last_login_time( void )
{
   Session->last_login_time=time(NULL);
}

time_t session_get_last_login_time( void )
{
   return Session->last_login_time;
}

void session_reset_last_login_time( void )
{
   Session->last_login_time=-1;
}

int session_check_login( void )
{
    time_t now;
    now = time(NULL);
    if ( Session -> challenge_sent == 1 ) {
        if ( (now-Session->last_login_time) > sys_config.max_last_login_time ) {
            if ( Session -> login_cnt > sys_config.max_login_cnt ) {
                return 0;
            }
        }
    }
    return 1;
}
