/*
 * Copyright (c) 2005 Kirk Murrin, <mailto:linuxguru1968@yahoo.com>
 * Licensed under the Gnu Public License, version 2.
 * All rights resered.
 * 
 */

#ifndef _SESSION_H
#define _SESSION_H

#include "mconfig.h"

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

typedef enum enum_auth_status {CHALLENGE_SENT,AUTH,NOTAUTH} nauth_status_type;

typedef struct struct_auth
{
   char *scheme;
   char *auth_data;
   char *usr_pass;
   char *username;
   char *password;
} auth_type;

typedef struct struct_auth_table  {
    char *scheme;
    int (*auth_handler)(void);
} auth_table_type;

typedef struct struct_session {
   char *authorization;
   int authenticated;
   int challenge_sent;
   int login_cnt;
   time_t last_login_time;
   auth_type auth_info;
   int addr_len;
   struct sockaddr_in client_addr;
   int port;
   int seq_num;
   char *session_id;
   long random_num;
   mstate_type     *mstate;
   block_type      *block;
   statement_type  *statement;
   char            *cwd;
} session_type;

void init_session(void);
int is_top_menu( void );
void clear_session( void );
char *set_session_id( void );
int check_sessionid( void );
void set_cwd( char *pathname );
void set_statement_default(void);
session_type *get_session( void );
mstate_type *get_session_mstate( void );
void init_stack( void );
void push_statement( void );
void pop_statement( void );
void pop_to_top( void );
void free_stack_entry( mstate_type  *mstate);
int session_challenge_sent( void );
int session_authenticated( void );
void session_set_challenge_sent( int sent );
void session_set_authenticated( int authenticated );

//int authenticate_session( int sd );
int process_authenticate( void );
int verify_credentials( void );
int auth_basic(void);

block_type *session_get_block_top( void );
statement_type *session_get_statement( void );
int session_set_statement( char *pointer_str );
void session_set_statement_block_first( void );
char *session_get_cwd( void );
long get_session_s_addr( void );

//void session_statement_sb_symbol_value(piece_type piece );
//void session_statement_sb_symbol_origin(origin_type origin_statement );

void session_update_login_cnt( void );
int session_get_login_cnt( void );
void session_reset_login_cnt( void );
void session_set_last_login_time( void );
time_t session_get_last_login_time( void );
void session_reset_last_login_time( void );
int session_check_login( void );


#endif /* */
