/*
 * Copyright (c) 2005 Kirk Murrin, <mailto:linuxguru1968@yahoo.com>
 * Licensed under the Gnu Public License, version 2.
 * All rights resered.
 * 
 */

#include <stdio.h>
#include <time.h>
#include <utmp.h>
#include <sys/utsname.h>
#include <sys/sysinfo.h>

#include "mconfig.h"
#include "html.h"
#include "response.h"
#include "request.h"

#include "status_page.h"

extern html_config_type page_config;
extern struct utsname Utsname;

#define STATUS_PAGE_ACTION_REFRESH "Refresh"


page_type status_page = {
        "System Status",	
        "&nbsp;",
        "&nbsp;",
        handle_status, 
        {
             INPUT,   { "submit",    PARAM_ACTION, STATUS_PAGE_ACTION_REFRESH },
             NONE,{ "",          "",          "",          }
        }
};


int handle_status( void )
{
   char *method,*action;
   action = get_request_action();
   method = get_request_method();

   set_response_title(NULL);
   if ( strcasecmp(method,"GET") == 0 ) {
   } else {
       if ( strcasecmp(method,"POST") == 0 ) {
           if ( strcmp(action,STATUS_PAGE_ACTION_REFRESH) == 0 ) {
           }
       }
   }
   setup_status();
}

#ifndef UT_IDSIZE
#define UT_IDSIZE 4            /* XXX - this is sizeof(struct utmp.ut_id) */
#endif

#define FSHIFT          16              /* nr of bits of precision */
#define FIXED_1         (1<<FSHIFT)     /* 1.0 as fixed-point */
#define LOAD_INT(x) ((x) >> FSHIFT)
#define LOAD_FRAC(x) LOAD_INT(((x) & (FIXED_1-1)) * 100)

int setup_status( void )
{
    int ret=0;
    FILE* fp;
    double uptime, idle_time;
    long utime, itime;
    const long minute = 60;
    const long hour = minute * 60;
    const long day = hour * 24;
    time_t curtime;
    struct tm *gm_time;
    time_t temp;
    struct utmp u_tmp, *u_tmp_p;
    struct sysinfo info;

    curtime = time (NULL);
    gm_time = gmtime(&curtime);

    //set_page_title("%s",Pages[STATUS]->title);
    set_response_title(NULL);

    start_page();
    start_body();

    start_content();

    start_div(page_config.max_textarea_cols*
              page_config.char_pixel_width,
              page_config.max_textarea_rows*
              page_config.char_pixel_height);

    start_table(1,0,0,0,page_config.table_bg_color,0);

    fp = fopen ("/proc/uptime", "r");

    if ( fp ) {

        fscanf (fp, "%lf %lf\n", &uptime, &idle_time);
        fclose (fp);

        utime = (long)uptime;
        itime = (long)idle_time;

        start_row_col_new(page_config.table_row_height,page_config.table_col_width);
        append_page_data_body("Up Time:\n");
        end_col();
        start_col();
        append_page_data_body("%ld days, %ld:%02ld:%02ld\n",utime/day, 
             (utime%day)/hour,(utime % hour)/minute,utime%minute);
        end_row_col();

        start_row_col_new(page_config.table_row_height,page_config.table_col_width);
        append_page_data_body("Idle Time:\n");
        end_col();
        start_col();
        append_page_data_body("%ld days, %ld:%02ld:%02ld\n",itime/day, 
             (itime%day)/hour,(itime % hour)/minute,itime%minute);
        end_row_col();

    } else {
        start_row_col_new(page_config.table_row_height,page_config.table_col_width);
        append_page_data_body("<b>/proc/uptime not availible</b>\n");
        end_row_col();
    }


    if ( strlen(Utsname.sysname) > 0 ) {
        start_row_col_new(page_config.table_row_height,page_config.table_col_width);
        append_page_data_body("Sysname:\n");
        end_col();
        start_col();
        append_page_data_body("%s\n",Utsname.sysname);
        end_row_col();

        start_row_col_new(page_config.table_row_height,page_config.table_col_width);
        append_page_data_body("Nodename:\n");
        end_col();
        start_col();
        append_page_data_body("%s\n",Utsname.nodename);
        end_row_col();

        start_row_col_new(page_config.table_row_height,page_config.table_col_width);
        append_page_data_body("Release:\n");
        end_col();
        start_col();
        append_page_data_body("%s\n",Utsname.release);
        end_row_col();

        start_row_col_new(page_config.table_row_height,page_config.table_col_width);
        append_page_data_body("Machine:\n");
        end_col();
        start_col();
        append_page_data_body("%s\n",Utsname.machine);
        end_row_col();

#ifdef _GNU_SOURCE
        start_row_col_new(page_config.table_row_height,page_config.table_col_width);
        append_page_data_body("Domainname:\n");
        end_col();
        start_col();
        append_page_data_body("%s\n",Utsname.domainname);
        end_row_col();
#endif

    }

    start_row_col_new(page_config.table_row_height,page_config.table_col_width);
    append_page_data_body("&nbsp;");
    end_row_col();

    memset(&info,0x0,sizeof(info));
    ret=sysinfo(&info);

    start_row_col_new(page_config.table_row_height,page_config.table_col_width);
    append_page_data_body("Load Average:");
    end_col();start_col();
    append_page_data_body("%ld.%02ld&#44; %ld.%02ld&#44; %ld.%02ld", 
			LOAD_INT(info.loads[0]), LOAD_FRAC(info.loads[0]), 
			LOAD_INT(info.loads[1]), LOAD_FRAC(info.loads[1]), 
			LOAD_INT(info.loads[2]), LOAD_FRAC(info.loads[2]));
    end_row_col();

    start_row_col_new(page_config.table_row_height,page_config.table_col_width);
    append_page_data_body("No. Processes:");
    end_col();start_col();
    append_page_data_body("%d",info.procs); 
    end_row_col();

    start_row_col_new(page_config.table_row_height,page_config.table_col_width);
    append_page_data_body("Total RAM:");
    end_col();start_col();
    append_page_data_body("%ld",(info.totalram/info.mem_unit)); 
    end_row_col();

    start_row_col_new(page_config.table_row_height,page_config.table_col_width);
    append_page_data_body("Used RAM:");
    end_col();start_col();
    append_page_data_body("%ld",
    (info.totalram/info.mem_unit)-(info.freeram/info.mem_unit)); 
    end_row_col();

    start_row_col_new(page_config.table_row_height,page_config.table_col_width);
    append_page_data_body("Avail. RAM:");
    end_col();start_col();
    append_page_data_body("%ld",(info.freeram/info.mem_unit)); 
    end_row_col();

    start_row_col_new(page_config.table_row_height,page_config.table_col_width);
    append_page_data_body("Amt.Shared RAM:");
    end_col();start_col();
    append_page_data_body("%ld",(info.sharedram/info.mem_unit)); 
    end_row_col();

    start_row_col_new(page_config.table_row_height,page_config.table_col_width);
    append_page_data_body("RAM For Buffers:");
    end_col();start_col();
    append_page_data_body("%ld",(info.bufferram/info.mem_unit)); 
    end_row_col();

    start_row_col_new(page_config.table_row_height,page_config.table_col_width);
    append_page_data_body("Total Swap:");
    end_col();start_col();
    append_page_data_body("%ld",(info.totalswap/info.mem_unit)); 
    end_row_col();

    start_row_col_new(page_config.table_row_height,page_config.table_col_width);
    append_page_data_body("Used Swap:");
    end_col();start_col();
    append_page_data_body("%ld",
             (info.totalswap/info.mem_unit)-(info.freeswap/info.mem_unit)); 
    end_row_col();

    start_row_col_new(page_config.table_row_height,page_config.table_col_width);
    append_page_data_body("Free Swap:");
    end_col();start_col();
    append_page_data_body("%ld",(info.freeswap/info.mem_unit)); 
    end_row_col();

    start_row_col_new(page_config.table_row_height,page_config.table_col_width);
    append_page_data_body("Total High Mem:");
    end_col();start_col();
    append_page_data_body("%ld",(info.totalhigh/info.mem_unit)); 
    end_row_col();

    start_row_col_new(page_config.table_row_height,page_config.table_col_width);
    append_page_data_body("Avail High Mem:");
    end_col();start_col();
    append_page_data_body("%ld",(info.freehigh/info.mem_unit)); 
    end_row_col();

    start_row_col_new(page_config.table_row_height,page_config.table_col_width);
    append_page_data_body("Mem Unit size(bytes):");
    end_col();start_col();
    append_page_data_body("%ld",info.mem_unit); 
    end_row_col();


    start_row_col_new(page_config.table_row_height,page_config.table_col_width);
    append_page_data_body("&nbsp;");
    end_row_col();

    utmpname(_PATH_UTMP);
    setutent(); 

    start_row_new(page_config.table_row_height);
    add_heading("Type");
    add_heading("Line");
    add_heading("Id");
    add_heading("Username");
    add_heading("Host");
    add_heading("Address");
    end_row();

    while ((u_tmp_p = getutent()) != NULL) {
        static const char *entry_type[] = {
             "*not known*",
             "run level",
             "boot time",
             "new time",
             "old time",
             "init process",
             "login process",
             "user process",
             "dead process"
         };

         start_row_col_new(page_config.table_row_height,page_config.table_col_width);
	 if (u_tmp_p->ut_type > 0 && u_tmp_p->ut_type <= DEAD_PROCESS)
	     append_page_data_body("[%s]", entry_type[u_tmp_p->ut_type]);
	 else
	     append_page_data_body("[?type:%d]", u_tmp_p->ut_type);
	     append_page_data_body("<pid=%d>", u_tmp_p->ut_pid);
         end_col();

         start_col_new("top","",page_config.table_col_width,-1,-1,"");
#if UT_LINESIZE == 32
	 append_page_data_body("[%.32s]", u_tmp_p->ut_line);
#else
	 append_page_data_body("[%.12s]", u_tmp_p->ut_line);
#endif
         end_col();

         start_col_new("top","",page_config.table_col_width,-1,-1,"");
	 append_page_data_body("<%.4s>", u_tmp_p->ut_id);
	 temp = (time_t) u_tmp_p->ut_time;
	 append_page_data_body("<%.24s>", ctime(&temp));
         end_col();

         start_col_new("top","",page_config.table_col_width,-1,-1,"");

#if UT_NAMESIZE == 32
         append_page_data_body("[%.32s]", u_tmp_p->ut_user);
#else
	 append_page_data_body("[%.8s]", u_tmp_p->ut_user);
#endif
         end_col();

         start_col_new("top","",page_config.table_col_width,-1,-1,"");
#if UT_HOSTSIZE == 256
	 append_page_data_body("{%.256s}", u_tmp_p->ut_host);
#else
	 append_page_data_body("{%.16s}", u_tmp_p->ut_host);
#endif
         end_col();

         start_col_new("top","",page_config.table_col_width,-1,-1,"");
	 append_page_data_body("{%d.",  u_tmp_p->ut_addr & 0xFF);
	 append_page_data_body("%d.", (u_tmp_p->ut_addr >> 8) & 0xFF);
	 append_page_data_body("%d.", (u_tmp_p->ut_addr >> 16) & 0xFF);
	 append_page_data_body("%d}.", (u_tmp_p->ut_addr >> 24) & 0xFF);
         end_col();
         end_row();
    }

    endutent(); 

    end_table();
    end_div();

    end_content();

    end_body();
    end_page();

    return SEND_PAGE; 

}
